@@ -18,6 +18,20 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Copyright (C) 2007 S.Çağlar Onur <caglar@pardus.org.tr>
+# Copyright (C) 2013 Ikey Doherty <ikey@solusos.com>
+
+# Notes to PiSi based distribution maintainers
+# /etc/PackageKit/pisi.conf must contain a mapping of PiSi component to
+# PackageKit groups for correct operation, i.e.
+#   system.utils       = system
+#   desktop.gnome      = desktop-gnome
+# If you have a BTS you must also provide Bug-Regex and Bug-URI fields, i.e:
+#   Bug-Regex = Bug-SolusOS: T(\d+)
+#   Bug-URI = http://inf.solusos.com/T%s
+# We use simple python string formatting to replace the %s with the first
+# matched group in the regular expression. So in the example above, we expect
+# to see "Bug-SolusOS: T9" for example, on its own line in a package update
+# comment.
 
 import pisi
 import pisi.ui
@@ -26,21 +40,25 @@ from packagekit.package import PackagekitPackage
 from packagekit import enums
 import os.path
 import piksemel
+import re
 
 class SimplePisiHandler(pisi.ui.UI):
-    
+
     def __init(self):
         pisi.ui.UI.__init__(self, False, False)
-        
-    def display_progress (self, **ka):
-        self.the_callback (**ka)
+
+    def display_progress(self, **ka):
+        self.the_callback(**ka)
+
 
 class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
-        
-    MAPPING_FILE = "/etc/PackageKit/pisi.conf"
+
+    SETTINGS_FILE = "/etc/PackageKit/pisi.conf"
 
     def __init__(self, args):
-        self._load_mapping_from_disk ()
+        self.bug_regex = None
+        self.bug_uri = None
+        self._load_settings()
         PackageKitBaseBackend.__init__(self, args)
 
         self.componentdb = pisi.db.componentdb.ComponentDB()
@@ -52,29 +70,40 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
         # Do not ask any question to users
         self.options = pisi.config.Options()
         self.options.yes_all = True
-        
+
         self.saved_ui = pisi.context.ui
 
-    def _load_mapping_from_disk (self):
+    def _load_settings(self):
         """ Load the PK Group-> PiSi component mapping """
-        if os.path.exists (self.MAPPING_FILE):
-            with open (self.MAPPING_FILE, "r") as mapping:
+        if os.path.exists(self.SETTINGS_FILE):
+            with open(self.SETTINGS_FILE, "r") as mapping:
                 self.groups = {}
                 for line in mapping.readlines():
-                    line = line.replace("\r","").replace("\n","").strip()
-                    if line.strip() == "" or "#" in line: continue
-                    splits = line.split ("=")
-                    pisi_component = splits[0].strip()
-                    pk_group = splits[1].strip()
-                    self.groups [pisi_component] = pk_group
+                    line = line.replace("\r", "").replace("\n", "").strip()
+                    if line.strip() == "" or "#" in line:
+                        continue
+
+                    splits = line.split("=")
+                    key = splits[0].strip()
+                    value = splits[1].strip()
+
+                    # Check if this contains our bug keys
+                    if key == "Bug-Regex":
+                        self.bug_regex = re.compile(value)
+                        continue
+                    if key == "Bug-URI":
+                        self.bug_uri = value
+                        continue
+                    self.groups[key] = value
         else:
             self.groups = {}
-                    
+
     def __get_package_version(self, package):
         """ Returns version string of given package """
         # Internal FIXME: PiSi may provide this
         if package.build is not None:
-            version = "%s-%s-%s" % (package.version, package.release, package.build)
+            version = "%s-%s-%s" % (package.version, package.release,
+                                    package.build)
         else:
             version = "%s-%s" % (package.version, package.release)
         return version
@@ -115,7 +144,8 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
         package = self.get_package_from_id(package_ids[0])[0]
 
         for pkg in self.packagedb.get_package(package).runtimeDependencies():
-            # Internal FIXME: PiSi API has really inconsistent for return types and arguments!
+            # FIXME: PiSi API has really inconsistent for return types
+            # and arguments!
             self.__get_package(pkg.package)
 
     def get_details(self, package_ids):
@@ -127,22 +157,21 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
 
         if self.packagedb.has_package(package):
             pkg = self.packagedb.get_package(package)
-            repo = self.packagedb.get_package_repo (pkg.name, None)
-            pkg_id = self.get_package_id (pkg.name, self.__get_package_version(pkg), pkg.architecture, repo[1])
+            repo = self.packagedb.get_package_repo(pkg.name, None)
+            pkg_id = self.get_package_id(pkg.name,
+                                         self.__get_package_version(pkg),
+                                         pkg.architecture, repo[1])
 
-            if self.groups.has_key(pkg.partOf):
+            if pkg.partOf in self.groups:
                 group = self.groups[pkg.partOf]
             else:
                 group = GROUP_UNKNOWN
-                
-            homepage = pkg.source.homepage if pkg.source.homepage is not None else ''
 
-            self.details(pkg_id,
-                            ",".join (pkg.license),
-                            group,
-                            pkg.description,
-                            homepage,
-                            pkg.packageSize)
+            homepage = pkg.source.homepage if pkg.source.homepage is not None\
+                else ''
+
+            self.details(pkg_id, ",".join(pkg.license), group, pkg.description,
+                         homepage, pkg.packageSize)
         else:
             self.error(ERROR_PACKAGE_NOT_FOUND, "Package was not found")
 
@@ -155,9 +184,11 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
 
         if self.installdb.has_package(package):
             pkg = self.packagedb.get_package(package)
-            repo = self.packagedb.get_package_repo (pkg.name, None)
-            pkg_id = self.get_package_id (pkg.name, self.__get_package_version(pkg), pkg.architecture, repo[1])
-            
+            repo = self.packagedb.get_package_repo(pkg.name, None)
+            pkg_id = self.get_package_id(pkg.name,
+                                         self.__get_package_version(pkg),
+                                         pkg.architecture, repo[1])
+
             pkg = self.installdb.get_files(package)
 
             files = map(lambda y: "/%s" % y.path, pkg.list)
@@ -173,7 +204,8 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
         for repo in pisi.api.list_repos():
             # Internal FIXME: What an ugly way to get repo uri
             # FIXME: Use repository enabled/disabled state
-            self.repo_detail(repo, self.repodb.get_repo(repo).indexuri.get_uri(), True)
+            uri = self.repodb.get_repo(repo).indexuri.get_uri()
+            self.repo_detail(repo, uri, True)
 
     def get_requires(self, filters, package_ids, recursive):
         """ Prints a list of requires for a given package """
@@ -191,101 +223,131 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
         self.allow_cancel(True)
         self.percentage(None)
 
+        self._updates = dict()
         for package in pisi.api.list_upgradable():
-
             pkg = self.packagedb.get_package(package)
-
             version = self.__get_package_version(pkg)
             id = self.get_package_id(pkg.name, version, pkg.architecture, "")
+            installed_package = self.installdb.get_package(package)
 
-            # Internal FIXME: PiSi must provide this information as a single API call :(
+            repo = self.packagedb.get_package_repo(pkg.name, None)[1]
+            pindex = "/var/lib/pisi/index/%s/pisi-index.xml" % repo
+
+            self._updates[pkg.name] = \
+                self._extract_update_details(pindex, pkg.name)
+            bug_uri = self._updates[pkg.name][3]
+
+            # FIXME: PiSi must provide this information as a single API call :(
             updates = [i for i in self.packagedb.get_package(package).history
-                    if pisi.version.Version(i.release) > pisi.version.Version(self.installdb.get_package(package).release)]
-            if pisi.util.any(lambda i:i.type == "security", updates):
+                       if pisi.version.Version(i.release) >
+                       installed_package.release]
+            if pisi.util.any(lambda i: i.type == "security", updates):
                 self.package(id, INFO_SECURITY, pkg.summary)
+            elif bug_uri != "":
+                self.package(id, INFO_BUGFIX, pkg.summary)
             else:
                 self.package(id, INFO_NORMAL, pkg.summary)
 
-    def _extract_update_details (self, pindex, package_name):
-        document = piksemel.parse (pindex)
-        packages = document.tags ("Package")
+    def _extract_update_details(self, pindex, package_name):
+        document = piksemel.parse(pindex)
+        packages = document.tags("Package")
         for pkg in packages:
-            if pkg.getTagData ("Name") == package_name:
+            if pkg.getTagData("Name") == package_name:
                 history = pkg.getTag("History")
-                update = history.tags ("Update")
+                update = history.tags("Update")
                 update_message = "Updated"
                 update_release = 0
-                update_data = ""
+                update_date = ""
+                needsReboot = False
+                bugURI = ""
                 for update in update:
-                    if int(update.getAttribute ("release")) > update_release:
-                        update_release = int(update.getAttribute ("release"))
-                        updater = update.getTagData ("Name")
-                        update_message = update.getTagData ("Comment")
-                        update_message = update_message.replace ("\n\n", ";").replace ("\n", " ")
-                        update_date = update.getTagData ("Date")
-                return (update_message,update_date)
-            pkg = pkg.nextTag ("Package")
-        return ("Log not found", "")
-		
+                    if int(update.getAttribute("release")) > update_release:
+                        update_release = int(update.getAttribute("release"))
+                        # updater = update.getTagData("Name")
+                        update_message = update.getTagData("Comment")
+                        update_message = update_message.replace("\n", ";")
+                        update_date = update.getTagData("Date")
+                        needsReboot = False
+                        try:
+                            requires = update.getTag("Requires")
+                            action = requires.getTagData("Action")
+                            if action == "systemRestart":
+                                needsReboot = True
+                        except Exception:
+                            pass
+                # Determine if this is a bug fix
+                for line in update_message.split(";"):
+                    m = self.bug_regex.match(line)
+                    if m is not None:
+                        bugURI = self.bug_uri % m.group(1)
+                        break
+                return (update_message, update_date, needsReboot, bugURI)
+            pkg = pkg.nextTag("Package")
+        return("Log not found", "", False, "")
+
     def get_update_detail(self, package_ids):
         for package_id in package_ids:
-            package = self.get_package_from_id (package_id)[0]
-            the_package = self.installdb.get_package (package)
+            package = self.get_package_from_id(package_id)[0]
+            the_package = self.installdb.get_package(package)
             updates = [package_id]
             obsoletes = ""
-            bugzilla_url = "" # TODO: Add regex matching for #FIXES:ID or something similar
+            # TODO: Add regex matching for #FIXES:ID or something similar
             cve_url = ""
             package_url = the_package.source.homepage
             vendor_url = package_url if package_url is not None else ""
             issued = ""
-            repo = self.packagedb.get_package_repo (package, None)[1]
-            pindex = "/var/lib/pisi/index/%s/pisi-index.xml" % repo
 
-            changelog = ""  
-            issued = updated = "" # TODO: Set to security_issued if security update     
-            update_message,security_issued = self._extract_update_details (pindex, package)
-            state = UPDATE_STATE_STABLE # TODO: Add tagging to repo's, or a mapping file
-            
+            changelog = ""
+            # TODO: Set to security_issued if security update
+            issued = updated = ""
+            update_message, security_issued, needsReboot, bugURI = \
+                self._updates[package]
+
+            # TODO: Add tagging to repo's, or a mapping file
+            state = UPDATE_STATE_STABLE
+            reboot = "system" if needsReboot else "none"
+
             self.update_detail(package_id, updates, obsoletes, vendor_url,
-                bugzilla_url, cve_url, "none", update_message, changelog,
-                state, issued, updated)
+                               bugURI, cve_url, reboot, update_message,
+                               changelog, state, issued, updated)
 
     def download_packages(self, directory, package_ids):
         """ Download the given packages to a directory """
-        self.allow_cancel (False)
-        self.percentage (None)
-        self.status (STATUS_DOWNLOAD)
+        self.allow_cancel(False)
+        self.percentage(None)
+        self.status(STATUS_DOWNLOAD)
 
         packages = list()
 
-        def progress_cb (**kw):
-            self.percentage (int(kw['percent']))
-            
-        ui = SimplePisiHandler ()
+        def progress_cb(**kw):
+            self.percentage(int(kw['percent']))
+
+        ui = SimplePisiHandler()
         for package_id in package_ids:
-            package = self.get_package_from_id (package_id)[0]
-            packages.append (package)
+            package = self.get_package_from_id(package_id)[0]
+            packages.append(package)
             try:
-                pkg = self.packagedb.get_package (package)
+                pkg = self.packagedb.get_package(package)
             except:
                 self.error(ERROR_PACKAGE_NOT_FOUND, "Package was not found")
         try:
-            pisi.api.set_userinterface (ui)
+            pisi.api.set_userinterface(ui)
             ui.the_callback = progress_cb
             if directory is None:
                 directory = os.path.curdir
-            pisi.api.fetch (packages, directory)
+            pisi.api.fetch(packages, directory)
             # Scan for package
             for package in packages:
-                package_obj = self.packagedb.get_package (package)
+                package_obj = self.packagedb.get_package(package)
                 uri = package_obj.packageURI.split("/")[-1]
-                location = os.path.join (directory, uri)
-                self.files (package_id, location)
-            pisi.api.set_userinterface (self.saved_ui)
+                location = os.path.join(directory, uri)
+                self.files(package_id, location)
+            pisi.api.set_userinterface(self.saved_ui)
         except Exception, e:
-            self.error(ERROR_PACKAGE_DOWNLOAD_FAILED, "Could not download package: %s" % e)
-        self.percentage (None)        
-            
+            self.error(ERROR_PACKAGE_DOWNLOAD_FAILED,
+                       "Could not download package: %s" % e)
+        self.percentage(None)
+
     def install_files(self, only_trusted, files):
         """ Installs given package into system"""
 
@@ -295,45 +357,49 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
         self.allow_cancel(False)
         self.percentage(None)
 
-        def progress_cb (**kw):			
-            self.percentage (int(kw['percent']))
-            
-        ui = SimplePisiHandler ()
+        def progress_cb(**kw):
+            self.percentage(int(kw['percent']))
+
+        ui = SimplePisiHandler()
 
         self.status(STATUS_INSTALL)
-        pisi.api.set_userinterface (ui)
+        pisi.api.set_userinterface(ui)
         ui.the_callback = progress_cb
 
         try:
             self.status(STATUS_INSTALL)
             pisi.api.install(files)
-        except pisi.Error,e:
+        except pisi.Error, e:
             # FIXME: Error: internal-error : Package re-install declined
             # Force needed?
             self.error(ERROR_PACKAGE_ALREADY_INSTALLED, e)
-        pisi.api.set_userinterface (self.saved_ui)
+        pisi.api.set_userinterface(self.saved_ui)
 
-    def _report_all_for_package (self, package, remove=False):
+    def _report_all_for_package(self, package, remove=False):
         """ Report all deps for the given package """
         if not remove:
             deps = self.packagedb.get_package(package).runtimeDependencies()
             # TODO: Add support to report conflicting packages requiring removal
             #conflicts = self.packagedb.get_package (package).conflicts
             for dep in deps:
-                if not self.installdb.has_package (dep.name()):
-                    dep_pkg = self.packagedb.get_package (dep.name())
-                    repo = self.packagedb.get_package_repo (dep_pkg.name, None)
-                    pkg_id = self.get_package_id (dep_pkg.name, self.__get_package_version(dep_pkg), dep_pkg.architecture, repo[1])
-                    self.package (pkg_id, INFO_INSTALLING, dep_pkg.summary)
+                if not self.installdb.has_package(dep.name()):
+                    dep_pkg = self.packagedb.get_package(dep.name())
+                    repo = self.packagedb.get_package_repo(dep_pkg.name, None)
+                    version = self.__get_package_version(dep_pkg)
+                    pkg_id = self.get_package_id(dep_pkg.name, version,
+                                                 dep_pkg.architecture, repo[1])
+                    self.package(pkg_id, INFO_INSTALLING, dep_pkg.summary)
         else:
             rev_deps = self.installdb.get_rev_deps(package)
             for rev_dep, depinfo in rev_deps:
-                if self.installdb.has_package (rev_dep):
-                    dep_pkg = self.packagedb.get_package (rev_dep)
-                    repo = self.packagedb.get_package_repo (dep_pkg.name, None)
-                    pkg_id = self.get_package_id (dep_pkg.name, self.__get_package_version(dep_pkg), dep_pkg.architecture, repo[1])
-                    self.package (pkg_id, INFO_REMOVING, dep_pkg.summary)
-                                                        
+                if self.installdb.has_package(rev_dep):
+                    dep_pkg = self.packagedb.get_package(rev_dep)
+                    repo = self.packagedb.get_package_repo(dep_pkg.name, None)
+                    version = self.__get_package_version(dep_pkg)
+                    pkg_id = self.get_package_id(dep_pkg.name, version,
+                                                 dep_pkg.architecture, repo[1])
+                    self.package(pkg_id, INFO_REMOVING, dep_pkg.summary)
+
     def install_packages(self, transaction_flags, package_ids):
         """ Installs given package into system"""
         # FIXME: fetch/install progress
@@ -344,31 +410,31 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
 
         # FIXME: use only_trusted
         for package_id in package_ids:
-            package = self.get_package_from_id (package_id)[0]
-            if self.installdb.has_package (package):
-                self.error (ERROR_PACKAGE_NOT_INSTALLED, "Package is already installed")
-            packages.append (package)
+            package = self.get_package_from_id(package_id)[0]
+            if self.installdb.has_package(package):
+                self.error(ERROR_PACKAGE_NOT_INSTALLED,
+                           "Package is already installed")
+            packages.append(package)
 
-        def progress_cb (**kw):			
-            self.percentage (int(kw['percent']))
-            
-        ui = SimplePisiHandler ()
+        def progress_cb(**kw):
+            self.percentage(int(kw['percent']))
+
+        ui = SimplePisiHandler()
 
         self.status(STATUS_INSTALL)
-        pisi.api.set_userinterface (ui)
+        pisi.api.set_userinterface(ui)
         ui.the_callback = progress_cb
 
         if TRANSACTION_FLAG_SIMULATE in transaction_flags:
             # Simulated, not real.
             for package in packages:
-                self._report_all_for_package (package)
+                self._report_all_for_package(package)
             return
         try:
             pisi.api.install(packages)
-        except pisi.Error,e:
+        except pisi.Error, e:
             self.error(ERROR_UNKNOWN, e)
-        pisi.api.set_userinterface (self.saved_ui)
-
+        pisi.api.set_userinterface(self.saved_ui)
 
     def refresh_cache(self, force):
         """ Updates repository indexes """
@@ -377,7 +443,7 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
         self.percentage(0)
         self.status(STATUS_REFRESH_CACHE)
 
-        slice = (100/len(pisi.api.list_repos()))/2
+        slice = (100 / len(pisi.api.list_repos())) / 2
 
         percentage = 0
         for repo in pisi.api.list_repos():
@@ -387,37 +453,39 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
 
         self.percentage(100)
 
-    def remove_packages(self, transaction_flags, package_ids, allowdeps, autoremove):
+    def remove_packages(self, transaction_flags, package_ids,
+                        allowdeps, autoremove):
         """ Removes given package from system"""
         self.allow_cancel(False)
         self.percentage(None)
         # TODO: use autoremove
         packages = list()
-        
+
         for package_id in package_ids:
-            package = self.get_package_from_id (package_id)[0]
-            if not self.installdb.has_package (package):
-                self.error (ERROR_PACKAGE_NOT_INSTALLED, "Package is not installed")
-            packages.append (package)
-    
-        def progress_cb (**kw):			
-            self.percentage (int(kw['percent']))
-            
-        ui = SimplePisiHandler ()
-        
+            package = self.get_package_from_id(package_id)[0]
+            if not self.installdb.has_package(package):
+                self.error(ERROR_PACKAGE_NOT_INSTALLED,
+                           "Package is not installed")
+            packages.append(package)
+
+        def progress_cb(**kw):
+            self.percentage(int(kw['percent']))
+
+        ui = SimplePisiHandler()
+
         package = self.get_package_from_id(package_ids[0])[0]
-        self.status (STATUS_REMOVE)
+        self.status(STATUS_REMOVE)
 
         if TRANSACTION_FLAG_SIMULATE in transaction_flags:
             # Simulated, not real.
             for package in packages:
-                self._report_all_for_package (package, remove=True)
+                self._report_all_for_package(package, remove=True)
             return
         try:
             pisi.api.remove(packages)
-        except pisi.Error,e:
+        except pisi.Error, e:
             self.error(ERROR_CANNOT_REMOVE_SYSTEM_PACKAGE, e)
-        pisi.api.set_userinterface (self.saved_ui)
+        pisi.api.set_userinterface(self.saved_ui)
 
     def repo_set_data(self, repo_id, parameter, value):
         """ Sets a parameter for the repository specified """
@@ -434,7 +502,8 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
                 pisi.api.update_repo(repo_id)
             except pisi.fetcher.FetchError:
                 pisi.api.remove_repo(repo_id)
-                self.error(ERROR_REPO_NOT_FOUND, "Could not reach the repository, removing from system")
+                err = "Could not reach the repository, removing from system"
+                self.error(ERROR_REPO_NOT_FOUND, err)
         elif parameter == "remove-repo":
             try:
                 pisi.api.remove_repo(repo_id)
@@ -444,7 +513,8 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
             self.error(ERROR_NOT_SUPPORTED, "Parameter not supported")
 
     def resolve(self, filters, package):
-        """ Turns a single package name into a package_id suitable for the other methods """
+        """ Turns a single package name into a package_id
+        suitable for the other methods """
         self.allow_cancel(True)
         self.percentage(None)
 
@@ -456,7 +526,8 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
         self.percentage(None)
         self.status(STATUS_INFO)
 
-        # Internal FIXME: Use search_details instead of _package when API gains that ability :)
+        # Internal FIXME: Use search_details instead of _package when API
+        # gains that ability :)
         for pkg in pisi.api.search_package(values):
             self.__get_package(pkg, filters)
 
@@ -480,16 +551,17 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
         self.status(STATUS_INFO)
 
         for value in values:
-			packages = list()
-			for item in self.groups:
-				if self.groups[item] == value:		
-					try:
-						packages.extend (self.componentdb.get_packages(item, walk=False))
-					except:
-						self.error(ERROR_GROUP_NOT_FOUND,
-								   "Component %s was not found" % value)
-			for pkg in packages:
-				self.__get_package(pkg, filters)
+            packages = list()
+            for item in self.groups:
+                if self.groups[item] == value:
+                    try:
+                        pkgs = self.componentdb.get_packages(item, walk=False)
+                        packages.extend(pkgs)
+                    except:
+                        self.error(ERROR_GROUP_NOT_FOUND,
+                                   "Component %s was not found" % value)
+            for pkg in packages:
+                self.__get_package(pkg, filters)
 
     def search_name(self, filters, values):
         """ Prints a list of packages contains search term in its name """
@@ -509,37 +581,35 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
         # FIXME: fetch/install progress
         self.allow_cancel(False)
         self.percentage(None)
-        
+
         packages = list()
         for package_id in package_ids:
-            package = self.get_package_from_id (package_id)[0]
-            if not self.installdb.has_package (package):
-                self.error (ERROR_PACKAGE_NOT_INSTALLED, "Cannot update a package that is not installed")
-            packages.append (package)
+            package = self.get_package_from_id(package_id)[0]
+            if not self.installdb.has_package(package):
+                self.error(ERROR_PACKAGE_NOT_INSTALLED,
+                           "Cannot update a package that is not installed")
+            packages.append(package)
 
-        def progress_cb (**kw):			
-            self.percentage (int(kw['percent']))
+        def progress_cb(**kw):
+            self.percentage(int(kw['percent']))
 
-        ui = SimplePisiHandler ()
-        pisi.api.set_userinterface (ui)
+        ui = SimplePisiHandler()
+        pisi.api.set_userinterface(ui)
         ui.the_callback = progress_cb
-        
+
         if TRANSACTION_FLAG_SIMULATE in transaction_flags:
             for package in packages:
-                self._report_all_for_package (package)
+                self._report_all_for_package(package)
             return
         try:
             pisi.api.upgrade(packages)
-        except pisi.Error,e:
+        except pisi.Error, e:
             self.error(ERROR_UNKNOWN, e)
-        pisi.api.set_userinterface (self.saved_ui)
-
+        pisi.api.set_userinterface(self.saved_ui)
 
     def update_system(self, only_trusted):
         """ Updates all available packages """
-
         # FIXME: use only_trusted
-
         # FIXME: fetch/install progress
         self.allow_cancel(False)
         self.percentage(None)
@@ -549,13 +619,13 @@ class PackageKitPisiBackend(PackageKitBaseBackend, PackagekitPackage):
 
         try:
             pisi.api.upgrade(pisi.api.list_upgradable())
-        except pisi.Error,e:
+        except pisi.Error, e:
             self.error(ERROR_UNKNOWN, e)
 
+
 def main():
     backend = PackageKitPisiBackend('')
     backend.dispatcher(sys.argv[1:])
 
 if __name__ == "__main__":
     main()
-
