@@ -80,25 +80,6 @@ pk_monitor_notify_network_status_cb (PkControl *control, GParamSpec *pspec, gpoi
 	g_print ("network status=%s\n", pk_network_enum_to_string (state));
 }
 
-/**
- * pk_monitor_message_cb:
- **/
-static void
-pk_monitor_message_cb (PkMessage *item, const gchar *transaction_id)
-{
-	gchar *details;
-	PkMessageEnum type;
-
-	/* get data */
-	g_object_get (item,
-		      "details", &details,
-		      "type", &type,
-		      NULL);
-
-	g_print ("%s\tmessage: %s, %s\n", transaction_id, pk_message_enum_to_string (type), details);
-	g_free (details);
-}
-
 /**
  * pk_monitor_media_change_required_cb:
  **/
@@ -134,7 +115,6 @@ pk_monitor_adopt_cb (PkClient *_client, GAsyncResult *res, gpointer user_data)
 	PkExitEnum exit_enum;
 	gchar *transaction_id = NULL;
 	PkError *error_code = NULL;
-	GPtrArray *message_array = NULL;
 	GPtrArray *media_array = NULL;
 
 	/* get the results */
@@ -158,10 +138,6 @@ pk_monitor_adopt_cb (PkClient *_client, GAsyncResult *res, gpointer user_data)
 	exit_enum = pk_results_get_exit_code (results);
 	g_print ("%s\texit code: %s\n", transaction_id, pk_exit_enum_to_string (exit_enum));
 
-	/* message */
-	message_array = pk_results_get_message_array (results);
-	g_ptr_array_foreach (message_array, (GFunc) pk_monitor_message_cb, transaction_id);
-
 	/* media change required */
 	media_array = pk_results_get_media_change_required_array (results);
 	g_ptr_array_foreach (media_array, (GFunc) pk_monitor_media_change_required_cb, transaction_id);
@@ -172,8 +148,6 @@ pk_monitor_adopt_cb (PkClient *_client, GAsyncResult *res, gpointer user_data)
 		g_print ("%s\terror code: %s, %s\n", transaction_id, pk_error_enum_to_string (pk_error_get_code (error_code)), pk_error_get_details (error_code));
 out:
 	g_free (transaction_id);
-	if (message_array != NULL)
-		g_ptr_array_unref (message_array);
 	if (media_array != NULL)
 		g_ptr_array_unref (media_array);
 	if (error_code != NULL)
