@@ -71,7 +71,7 @@ pk_cnf_find_alternatives_swizzle (const gchar *cmd, guint len, GPtrArray *array)
 	gchar swap;
 
 	/*  */
-	for (i=0; i<len-1; i++) {
+	for (i = 0; i < len-1; i++) {
 		possible = g_strdup (cmd);
 		swap = possible[i];
 		possible[i] = possible[i+1];
@@ -93,7 +93,7 @@ pk_cnf_find_alternatives_replace (const gchar *cmd, guint len, GPtrArray *array)
 	gchar temp;
 
 	/* replace some easily confused chars */
-	for (i=0; i<len; i++) {
+	for (i = 0; i < len; i++) {
 		temp = cmd[i];
 		if (temp == 'i') {
 			possible = g_strdup (cmd);
@@ -161,7 +161,7 @@ pk_cnf_find_alternatives_truncate (const gchar *cmd, guint len, GPtrArray *array
 
 	/* truncate first char */
 	possible = g_strdup (cmd);
-	for (i=0; i<len-1; i++)
+	for (i = 0; i < len-1; i++)
 		possible[i] = possible[i+1];
 	possible[len-1] = '\0';
 	g_ptr_array_add (array, possible);
@@ -211,6 +211,84 @@ pk_cnf_find_alternatives_locale (const gchar *cmd, guint len, GPtrArray *array)
 	}
 }
 
+/**
+ * pk_cnf_find_alternatives_solaris:
+ *
+ * Suggest Linux commands for Solaris commands
+ **/
+static void
+pk_cnf_find_alternatives_solaris (const gchar *cmd, guint len, GPtrArray *array)
+{
+	GHashTable *hash;
+	const gchar *tmp;
+
+	hash = g_hash_table_new (g_str_hash, g_str_equal);
+	g_hash_table_insert (hash, (gpointer) "smuser", (gpointer) "usermod");
+	g_hash_table_insert (hash, (gpointer) "logins", (gpointer) "usermod");
+	g_hash_table_insert (hash, (gpointer) "adb", (gpointer) "gdb");
+	g_hash_table_insert (hash, (gpointer) "add_drv", (gpointer) "modprobe");
+	g_hash_table_insert (hash, (gpointer) "modload", (gpointer) "modprobe");
+	g_hash_table_insert (hash, (gpointer) "modunload", (gpointer) "modprobe");
+	g_hash_table_insert (hash, (gpointer) "rem_drv", (gpointer) "modprobe");
+	g_hash_table_insert (hash, (gpointer) "audit", (gpointer) "auditctl");
+	g_hash_table_insert (hash, (gpointer) "auditreduce", (gpointer) "auditctl");
+	g_hash_table_insert (hash, (gpointer) "cfgadm", (gpointer) "lsmod");
+	g_hash_table_insert (hash, (gpointer) "clri", (gpointer) "fsck");
+	g_hash_table_insert (hash, (gpointer) "fsdb", (gpointer) "fsck");
+	g_hash_table_insert (hash, (gpointer) "volcheck", (gpointer) "fsck");
+	g_hash_table_insert (hash, (gpointer) "crle", (gpointer) "ldconfig");
+	g_hash_table_insert (hash, (gpointer) "devfsadm", (gpointer) "udevtrigger");
+	g_hash_table_insert (hash, (gpointer) "devlinks", (gpointer) "ln");
+	g_hash_table_insert (hash, (gpointer) "dfshares", (gpointer) "exportfs");
+	g_hash_table_insert (hash, (gpointer) "share", (gpointer) "exportfs");
+	g_hash_table_insert (hash, (gpointer) "shareall", (gpointer) "exportfs");
+	g_hash_table_insert (hash, (gpointer) "dladm", (gpointer) "ifconfig");
+	g_hash_table_insert (hash, (gpointer) "kstat", (gpointer) "ifconfig");
+	g_hash_table_insert (hash, (gpointer) "dtrace", (gpointer) "stap");
+	g_hash_table_insert (hash, (gpointer) "eeprom", (gpointer) "hwclock");
+	g_hash_table_insert (hash, (gpointer) "fcinfo", (gpointer) "lspci");
+	g_hash_table_insert (hash, (gpointer) "prtfru", (gpointer) "lspci");
+	g_hash_table_insert (hash, (gpointer) "fmthard", (gpointer) "fdisk");
+	g_hash_table_insert (hash, (gpointer) "format", (gpointer) "fdisk");
+	g_hash_table_insert (hash, (gpointer) "prtvtoc", (gpointer) "fdisk");
+	g_hash_table_insert (hash, (gpointer) "installboot", (gpointer) "mkbootdisk");
+	g_hash_table_insert (hash, (gpointer) "installpatch", (gpointer) "yum");
+	g_hash_table_insert (hash, (gpointer) "patchaddpkgadd", (gpointer) "yum");
+	g_hash_table_insert (hash, (gpointer) "pkgchk", (gpointer) "yum");
+	g_hash_table_insert (hash, (gpointer) "pkginfo", (gpointer) "yum");
+	g_hash_table_insert (hash, (gpointer) "pkgrm", (gpointer) "yum");
+	g_hash_table_insert (hash, (gpointer) "prodreg", (gpointer) "yum");
+	g_hash_table_insert (hash, (gpointer) "showrev", (gpointer) "yum");
+	g_hash_table_insert (hash, (gpointer) "isainfo", (gpointer) "uname");
+	g_hash_table_insert (hash, (gpointer) "luxadm", (gpointer) "systool");
+	g_hash_table_insert (hash, (gpointer) "mkfile", (gpointer) "touch");
+	g_hash_table_insert (hash, (gpointer) "mpathadm", (gpointer) "multipath");
+	g_hash_table_insert (hash, (gpointer) "stmsboot", (gpointer) "multipath");
+	g_hash_table_insert (hash, (gpointer) "ndd", (gpointer) "modinfo");
+	g_hash_table_insert (hash, (gpointer) "newfs", (gpointer) "mkfs");
+	g_hash_table_insert (hash, (gpointer) "pbind", (gpointer) "taskset");
+	g_hash_table_insert (hash, (gpointer) "pldd", (gpointer) "ldd");
+	g_hash_table_insert (hash, (gpointer) "praudit", (gpointer) "auditctl");
+	g_hash_table_insert (hash, (gpointer) "prstat", (gpointer) "ps");
+	g_hash_table_insert (hash, (gpointer) "prtconf", (gpointer) "dmesg");
+	g_hash_table_insert (hash, (gpointer) "psrinfo", (gpointer) "dmidecode");
+	g_hash_table_insert (hash, (gpointer) "sysdef", (gpointer) "dmidecode");
+	g_hash_table_insert (hash, (gpointer) "ptree", (gpointer) "pstree");
+	g_hash_table_insert (hash, (gpointer) "snoop", (gpointer) "tcpdump");
+	g_hash_table_insert (hash, (gpointer) "sotruss", (gpointer) "strace");
+	g_hash_table_insert (hash, (gpointer) "truss", (gpointer) "strace");
+	g_hash_table_insert (hash, (gpointer) "svcadm", (gpointer) "service");
+	g_hash_table_insert (hash, (gpointer) "svcs", (gpointer) "service");
+	g_hash_table_insert (hash, (gpointer) "swap", (gpointer) "swapon");
+	g_hash_table_insert (hash, (gpointer) "trapstat", (gpointer) "oprofile");
+
+	/* find anything that matches exactly */
+	tmp = g_hash_table_lookup (hash, cmd);
+	if (tmp != NULL)
+		g_ptr_array_add (array, g_strdup (tmp));
+	g_hash_table_unref (hash);
+}
+
 /**
  * pk_cnf_find_alternatives_case:
  *
@@ -223,7 +301,7 @@ pk_cnf_find_alternatives_case (const gchar *cmd, guint len, GPtrArray *array)
 	gchar *possible;
 	gchar temp;
 
-	for (i=0; i<len; i++) {
+	for (i = 0; i < len; i++) {
 		temp = g_ascii_tolower (cmd[i]);
 		if (temp != cmd[i]) {
 			possible = g_strdup (cmd);
@@ -240,7 +318,7 @@ pk_cnf_find_alternatives_case (const gchar *cmd, guint len, GPtrArray *array)
 
 	/* all lower */
 	possible = g_strdup (cmd);
-	for (i=0; i<len; i++)
+	for (i = 0; i < len; i++)
 		possible[i] = g_ascii_tolower (cmd[i]);
 	if (strcmp (possible, cmd) != 0)
 		g_ptr_array_add (array, possible);
@@ -249,7 +327,7 @@ pk_cnf_find_alternatives_case (const gchar *cmd, guint len, GPtrArray *array)
 
 	/* all upper */
 	possible = g_strdup (cmd);
-	for (i=0; i<len; i++)
+	for (i = 0; i < len; i++)
 		possible[i] = g_ascii_toupper (cmd[i]);
 	if (strcmp (possible, cmd) != 0)
 		g_ptr_array_add (array, possible);
@@ -285,9 +363,10 @@ pk_cnf_find_alternatives (const gchar *cmd, guint len)
 	pk_cnf_find_alternatives_remove_double (cmd, len, possible);
 	pk_cnf_find_alternatives_case (cmd, len, possible);
 	pk_cnf_find_alternatives_locale (cmd, len, possible);
+	pk_cnf_find_alternatives_solaris (cmd, len, possible);
 
 	/* remove duplicates using a helper array */
-	for (i=0; i<possible->len; i++) {
+	for (i = 0; i < possible->len; i++) {
 		cmdt = g_ptr_array_index (possible, i);
 		ret = TRUE;
 		for (j=0; j<unique->len; j++) {
@@ -309,7 +388,7 @@ pk_cnf_find_alternatives (const gchar *cmd, guint len)
 	strncpy (buffer_sbin, "/usr/sbin/", PK_MAX_PATH_LEN);
 
 	/* remove any that exist (fast path) */
-	for (i=0; i<unique->len; i++) {
+	for (i = 0; i < unique->len; i++) {
 		cmdt = g_ptr_array_index (unique, i);
 
 		/* ITS4: ignore, size is checked */
@@ -435,8 +514,10 @@ pk_cnf_find_available (const gchar *cmd, guint max_search_time)
 	results = pk_client_search_files (PK_CLIENT(task), filters, values, cancellable,
 					  NULL, NULL, &error);
 	if (results == NULL) {
-		/* TRANSLATORS: we failed to find the package, this shouldn't happen */
-		g_printerr ("%s: %s\n", _("Failed to search for file"), error->message);
+		if (!g_error_matches (error, PK_CLIENT_ERROR, PK_CLIENT_ERROR_INVALID_INPUT)) {
+			/* TRANSLATORS: we failed to find the package, this shouldn't happen */
+			g_printerr ("%s: %s\n", _("Failed to search for file"), error->message);
+		}
 		g_error_free (error);
 		goto out;
 	}
@@ -458,7 +539,7 @@ pk_cnf_find_available (const gchar *cmd, guint max_search_time)
 	/* get the packages returned */
 	array = pk_results_get_package_array (results);
 	package_ids = g_new0 (gchar *, array->len+1);
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		item = g_ptr_array_index (array, i);
 		package_ids[i] = g_strdup (pk_package_get_id (item));
 	}
@@ -535,16 +616,17 @@ pk_cnf_get_config (void)
 	config->multiple_match = PK_CNF_POLICY_UNKNOWN;
 	config->single_install = PK_CNF_POLICY_UNKNOWN;
 	config->multiple_install = PK_CNF_POLICY_UNKNOWN;
-	config->software_source_search = FALSE;
-	config->similar_name_search = FALSE;
+	config->software_source_search = TRUE;
+	config->similar_name_search = TRUE;
 	config->locations = NULL;
+	config->max_search_time = 5000;
 
 	/* load file */
 	file = g_key_file_new ();
 	path = g_build_filename (SYSCONFDIR, "PackageKit", "CommandNotFound.conf", NULL);
 	ret = g_key_file_load_from_file (file, path, G_KEY_FILE_NONE, &error);
 	if (!ret) {
-		g_warning ("failed to open policy: %s", error->message);
+		g_printerr ("failed to load config file: %s\n", error->message);
 		g_error_free (error);
 		goto out;
 	}
@@ -664,6 +746,40 @@ pk_cnf_sigint_handler (int sig)
 	kill (getpid (), SIGINT);
 }
 
+/**
+ * pk_cnf_is_backend_fast_enough_to_do_search:
+**/
+static gboolean
+pk_cnf_is_backend_fast_enough_to_do_search (void)
+{
+	gboolean ret = FALSE;
+	gchar *backend;
+	GError *error = NULL;
+	PkControl *control = NULL;
+
+	/* Initialize PkControl */
+	control = pk_control_new ();
+	ret = pk_control_get_properties (control, cancellable, &error);
+	if (!ret) {
+		/* failed to contact the daemon */
+		g_error_free (error);
+		goto out;
+	}
+
+	/* Find backend name */
+	g_object_get (control, "backend-name", &backend, NULL);
+
+	/* Current list of too slow backends */
+	if (g_strcmp0 (backend, "yum") == 0) {
+		ret = FALSE;
+		goto out;
+	}
+out:
+	if (control != NULL)
+		g_object_unref(control);
+	return ret;
+}
+
 /**
  * main:
  **/
@@ -679,7 +795,7 @@ main (int argc, char *argv[])
 	gchar *text;
 	const gchar *possible;
 	gchar **parts;
-	guint retval = EXIT_SUCCESS;
+	guint retval = EXIT_COMMAND_NOT_FOUND;
 
 	setlocale (LC_ALL, "");
 	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
@@ -690,7 +806,9 @@ main (int argc, char *argv[])
 	if (! g_thread_supported ())
 		g_thread_init (NULL);
 #endif
+#if (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 35)
 	g_type_init ();
+#endif
 
 	/* don't show debugging, unless VERBOSE is specified */
 	pk_debug_add_log_domain (G_LOG_DOMAIN);
@@ -714,10 +832,8 @@ main (int argc, char *argv[])
 
 	/* get length */
 	len = strlen (argv[1]);
-	if (len < 1) {
-		retval = EXIT_COMMAND_NOT_FOUND;
+	if (len < 1)
 		goto out;
-	}
 
 	/* TRANSLATORS: the prefix of all the output telling the user
 	 * why it's not executing. NOTE: this is lowercase to mimic
@@ -727,7 +843,6 @@ main (int argc, char *argv[])
 	/* user is not allowing CNF to do anything useful */
 	if (!config->software_source_search &&
 	    !config->similar_name_search) {
-		retval = EXIT_COMMAND_NOT_FOUND;
 		goto out;
 	}
 
@@ -741,7 +856,6 @@ main (int argc, char *argv[])
 		if (config->single_match == PK_CNF_POLICY_WARN) {
 			/* TRANSLATORS: tell the user what we think the command is */
 			g_printerr ("%s '%s'\n", _("Similar command is:"), possible);
-			retval = EXIT_COMMAND_NOT_FOUND;
 			goto out;
 		}
 
@@ -758,8 +872,6 @@ main (int argc, char *argv[])
 			ret = pk_console_get_prompt (text, TRUE);
 			if (ret)
 				retval = pk_cnf_spawn_command (possible, &argv[2]);
-			else
-				retval = EXIT_COMMAND_NOT_FOUND;
 			g_free (text);
 		}
 		goto out;
@@ -769,7 +881,7 @@ main (int argc, char *argv[])
 		if (config->multiple_match == PK_CNF_POLICY_WARN) {
 			/* TRANSLATORS: show the user a list of commands that they could have meant */
 			g_printerr ("%s:\n", _("Similar commands are:"));
-			for (i=0; i<array->len; i++) {
+			for (i = 0; i < array->len; i++) {
 				possible = g_ptr_array_index (array, i);
 				g_printerr ("'%s'\n", possible);
 			}
@@ -778,7 +890,7 @@ main (int argc, char *argv[])
 		} else if (config->multiple_match == PK_CNF_POLICY_ASK) {
 			/* TRANSLATORS: show the user a list of commands we could run */
 			g_printerr ("%s:\n", _("Similar commands are:"));
-			for (i=0; i<array->len; i++) {
+			for (i = 0; i < array->len; i++) {
 				possible = g_ptr_array_index (array, i);
 				g_printerr ("%i\t'%s'\n", i+1, possible);
 			}
@@ -793,7 +905,8 @@ main (int argc, char *argv[])
 		goto out;
 
 	/* only search using PackageKit if configured to do so */
-	} else if (config->software_source_search) {
+	} else if (config->software_source_search &&
+		   pk_cnf_is_backend_fast_enough_to_do_search ()) {
 		package_ids = pk_cnf_find_available (argv[1], config->max_search_time);
 		if (package_ids == NULL)
 			goto out;
@@ -816,8 +929,6 @@ main (int argc, char *argv[])
 					ret = pk_cnf_install_package_id (package_ids[0]);
 					if (ret)
 						retval = pk_cnf_spawn_command (argv[1], &argv[2]);
-					else
-						retval = EXIT_COMMAND_NOT_FOUND;
 				}
 				g_print ("\n");
 				goto out;
@@ -828,8 +939,6 @@ main (int argc, char *argv[])
 				ret = pk_cnf_install_package_id (package_ids[0]);
 				if (ret)
 					retval = pk_cnf_spawn_command (argv[1], &argv[2]);
-				else
-					retval = EXIT_COMMAND_NOT_FOUND;
 			}
 			g_strfreev (parts);
 			goto out;
@@ -853,16 +962,17 @@ main (int argc, char *argv[])
 					g_strfreev (parts);
 				}
 
-				/* get selection */
 				/* TRANSLATORS: ask the user to choose a file to install */
 				i = pk_console_get_number (_("Please choose a package to install"), len);
+				if (i == 0) {
+					g_printerr ("%s\n", _("User aborted selection"));
+					goto out;
+				}
 
 				/* run command */
-				ret = pk_cnf_install_package_id (package_ids[i]);
+				ret = pk_cnf_install_package_id (package_ids[i - 1]);
 				if (ret)
 					retval = pk_cnf_spawn_command (argv[1], &argv[2]);
-				else
-					retval = EXIT_COMMAND_NOT_FOUND;
 			}
 			goto out;
 		}
