@@ -1,6 +1,7 @@
 # A plugin for yum which notifies PackageKit to refresh its data
 #
 # Copyright (c) 2007 James Bowes <jbowes@redhat.com>
+# Copyright (c) 2012 Elad Alfassa <elad@fedoraproject.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,3 +40,22 @@ def posttrans_hook(conduit):
         conduit.info(2, "Unable to send message to PackageKit")
         conduit.info(6, "%s" %(e,))
 
+def init_hook(conduit):
+    """
+    Tell PackageKit to exit when we start yum, so it won't block command-line operations.
+    """
+    try:
+        bus = dbus.SystemBus()
+    except dbus.DBusException, e:
+        conduit.info(2, "Unable to connect to dbus")
+        conduit.info(6, "%s" %(e,))
+        return
+    try:
+        packagekit_proxy = bus.get_object('org.freedesktop.PackageKit',
+                                          '/org/freedesktop/PackageKit')
+        packagekit_iface = dbus.Interface(packagekit_proxy, 'org.freedesktop.PackageKit')
+        packagekit_iface.SuggestDaemonQuit()
+    except Exception, e:
+        conduit.info(2, "Unable to send message to PackageKit")
+        conduit.info(6, "%s" %(e,))
+
