@@ -171,127 +171,6 @@ pk_console_get_prompt (const gchar *question, gboolean defaultyes)
 	return ret;
 }
 
-/**
- * pk_console_resolve_package:
- **/
-gchar *
-pk_console_resolve_package (PkClient *client, PkBitfield filter, const gchar *package_name, GError **error)
-{
-	const gchar *package_id_tmp;
-	gchar *package_id = NULL;
-	gboolean valid;
-	gchar **tmp;
-	gchar **split = NULL;
-	PkResults *results;
-	GPtrArray *array = NULL;
-	guint i;
-	gchar *printable;
-	PkPackage *package;
-	PkError *error_code = NULL;
-
-	/* have we passed a complete package_id? */
-	valid = pk_package_id_check (package_name);
-	if (valid)
-		return g_strdup (package_name);
-
-	/* split */
-	tmp = g_strsplit (package_name, ",", -1);
-
-	/* get the list of possibles */
-	results = pk_client_resolve (client, filter, tmp, NULL, NULL, NULL, error);
-	if (results == NULL)
-		goto out;
-
-	/* check error code */
-	error_code = pk_results_get_error_code (results);
-	if (error_code != NULL) {
-		g_set_error (error, 1, 0, "\n%s", pk_error_get_details (error_code));
-		goto out;
-	}
-
-	/* get the packages returned */
-	array = pk_results_get_package_array (results);
-	if (array == NULL) {
-		g_set_error (error, 1, 0, "did not get package struct for %s", package_name);
-		goto out;
-	}
-
-	/* nothing found */
-	if (array->len == 0) {
-		g_set_error (error, 1, 0, "could not find %s", package_name);
-		goto out;
-	}
-
-	/* just one thing found */
-	if (array->len == 1) {
-		package = g_ptr_array_index (array, 0);
-		g_object_get (package,
-			      "package-id", &package_id,
-			      NULL);
-		goto out;
-	}
-
-	/* TRANSLATORS: more than one package could be found that matched, to follow is a list of possible packages  */
-	g_print ("%s\n", _("More than one package matches:"));
-	for (i=0; i<array->len; i++) {
-		package = g_ptr_array_index (array, i);
-		package_id_tmp = pk_package_get_id (package);
-		split = pk_package_id_split (package_id_tmp);
-		printable = pk_package_id_to_printable (package_id_tmp);
-		g_print ("%i. %s [%s]\n", i+1, printable, split[PK_PACKAGE_ID_DATA]);
-		g_free (printable);
-	}
-
-	/* TRANSLATORS: This finds out which package in the list to use */
-	i = pk_console_get_number (_("Please choose the correct package: "), array->len);
-	if (i == 0) {
-		g_set_error_literal (error, 1, 0, "User aborted selection");
-		goto out;
-	}
-	package = g_ptr_array_index (array, i-1);
-	g_object_get (package,
-		      "package-id", &package_id,
-		      NULL);
-out:
-	if (results != NULL)
-		g_object_unref (results);
-	if (array != NULL)
-		g_ptr_array_unref (array);
-	g_strfreev (tmp);
-	g_strfreev (split);
-	return package_id;
-}
-
-/**
- * pk_console_resolve_packages:
- **/
-gchar **
-pk_console_resolve_packages (PkClient *client, PkBitfield filter, gchar **packages, GError **error)
-{
-	gchar **package_ids;
-	guint i;
-	guint len;
-
-	/* get length */
-	len = g_strv_length (packages);
-	g_debug ("resolving %i packages", len);
-
-	/* create output array*/
-	package_ids = g_new0 (gchar *, len+1);
-
-	/* resolve each package */
-	for (i=0; i<len; i++) {
-		package_ids[i] = pk_console_resolve_package (client, filter, packages[i], error);
-		if (package_ids[i] == NULL) {
-			/* destroy state */
-			g_strfreev (package_ids);
-			package_ids = NULL;
-			break;
-		}
-	}
-	return package_ids;
-}
-
 /**
  * pk_status_enum_to_localised_text:
  **/
