@@ -238,8 +238,8 @@ pk_package_sack_add_package (PkPackageSack *sack, PkPackage *package)
 	g_ptr_array_add (sack->priv->array,
 			 g_object_ref (package));
 	g_hash_table_insert (sack->priv->table,
-			     g_strdup (pk_package_get_id (package)),
-			     g_object_ref (package));
+			     (gpointer) pk_package_get_id (package),
+			     (gpointer) package);
 
 	return TRUE;
 }
@@ -555,6 +555,48 @@ pk_package_sack_find_by_id (PkPackageSack *sack, const gchar *package_id)
 	return package;
 }
 
+/**
+ * pk_package_sack_find_by_id_name_arch:
+ * @sack: a valid #PkPackageSack instance
+ * @package_id: a package_id descriptor
+ *
+ * Finds a package in a sack by package name and architecture. As soon as one
+ * package is found the search is stopped.
+ *
+ * Return value: (transfer full): the #PkPackage object, or %NULL if not found.
+ *
+ * Since: 0.8.16
+ */
+PkPackage *
+pk_package_sack_find_by_id_name_arch (PkPackageSack *sack, const gchar *package_id)
+{
+	PkPackage *pkg = NULL;
+	PkPackage *pkg_tmp;
+	gchar **split;
+	guint i;
+
+	g_return_val_if_fail (PK_IS_PACKAGE_SACK (sack), NULL);
+	g_return_val_if_fail (package_id != NULL, NULL);
+
+	/* does the package name feature in the array */
+	split = pk_package_id_split (package_id);
+	if (split == NULL)
+		goto out;
+	for (i = 0; i < sack->priv->array->len; i++) {
+		pkg_tmp = g_ptr_array_index (sack->priv->array, i);
+		if (g_strcmp0 (pk_package_get_name (pkg_tmp),
+			       split[PK_PACKAGE_ID_NAME]) == 0 &&
+		    g_strcmp0 (pk_package_get_arch (pkg_tmp),
+			       split[PK_PACKAGE_ID_ARCH]) == 0) {
+			pkg = g_object_ref (pkg_tmp);
+			break;
+		}
+	}
+out:
+	g_strfreev (split);
+	return pkg;
+}
+
 /**
  * pk_package_sack_sort_compare_name_func:
  **/
@@ -1210,7 +1252,7 @@ pk_package_sack_init (PkPackageSack *sack)
 	sack->priv = PK_PACKAGE_SACK_GET_PRIVATE (sack);
 	priv = sack->priv;
 
-	priv->table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+	priv->table = g_hash_table_new (g_str_hash, g_str_equal);
 	priv->array = g_ptr_array_new_with_free_func (g_object_unref);
 	priv->client = pk_client_new ();
 }
