@@ -50,7 +50,8 @@ struct _PkPackagePrivate
 {
 	PkInfoEnum		 info;
 	gchar			*package_id;
-	gchar			**package_id_split;
+	gchar			*package_id_data;
+	const gchar		*package_id_split[4];
 	gchar			*summary;
 	gchar			*license;
 	PkGroupEnum		 group;
@@ -158,40 +159,44 @@ pk_package_equal_id (PkPackage *package1, PkPackage *package2)
 gboolean
 pk_package_set_id (PkPackage *package, const gchar *package_id, GError **error)
 {
-	gboolean ret;
-	gchar **sections = NULL;
 	PkPackagePrivate *priv = package->priv;
+	gboolean ret;
+	guint cnt = 0;
+	guint i;
 
 	g_return_val_if_fail (PK_IS_PACKAGE (package), FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	/* check valid UTF8 */
-	ret = g_utf8_validate (package_id, -1, NULL);
-	if (!ret) {
-		g_set_error_literal (error, 1, 0, "invalid UTF8!");
-		goto out;
-	}
+	/* free old data */
+	g_free (priv->package_id);
+	g_free (priv->package_id_data);
 
-	/* split by delimeter */
-	sections = g_strsplit (package_id, ";", -1);
-	ret = (g_strv_length (sections) == 4);
-	if (!ret) {
-		g_set_error_literal (error, 1, 0, "invalid number of sections");
+	/* copy the package-id into package_id_data, change the ';' into '\0'
+	 * and reference the pointers in the const gchar * array */
+	priv->package_id = g_strdup (package_id);
+	priv->package_id_data = g_strdup (package_id);
+	priv->package_id_split[0] = priv->package_id_data;
+	for (i = 0; priv->package_id_data[i] != '\0'; i++) {
+		if (package_id[i] == ';') {
+			if (++cnt > 3)
+				continue;
+			priv->package_id_split[cnt] = &priv->package_id_data[i+1];
+			priv->package_id_data[i] = '\0';
+		}
+	}
+	if (cnt != 3) {
+		ret = FALSE;
+		g_set_error (error, 1, 0, "invalid number of sections %i", cnt);
 		goto out;
 	}
 
 	/* name has to be valid */
-	ret = (sections[0][0] != '\0');
+	ret = (priv->package_id_split[0][0] != '\0');
 	if (!ret) {
 		g_set_error_literal (error, 1, 0, "name invalid");
 		goto out;
 	}
-
-	/* save */
-	priv->package_id = g_strdup (package_id);
-	priv->package_id_split = g_strdupv (sections);
 out:
-	g_strfreev (sections);
 	return ret;
 }
 
@@ -221,6 +226,7 @@ pk_package_parse (PkPackage *package, const gchar *data, GError **error)
 	if (g_strv_length (sections) != 3) {
 		ret = FALSE;
 		g_set_error_literal (error, 1, 0, "data invalid");
+		goto out;
 	}
 
 	/* parse object */
@@ -252,6 +258,39 @@ pk_package_get_info (PkPackage *package)
 	return package->priv->info;
 }
 
+/**
+ * pk_package_set_info:
+ * @package: a valid #PkPackage instance
+ * @info: the %PkInfoEnum
+ *
+ * Sets the package info enum.
+ *
+ * Since: 0.8.14
+ **/
+void
+pk_package_set_info (PkPackage *package, PkInfoEnum info)
+{
+	g_return_if_fail (PK_IS_PACKAGE (package));
+	package->priv->info = info;
+}
+
+/**
+ * pk_package_set_summary:
+ * @package: a valid #PkPackage instance
+ * @summary: the package summary
+ *
+ * Sets the package summary.
+ *
+ * Since: 0.8.14
+ **/
+void
+pk_package_set_summary (PkPackage *package, const gchar *summary)
+{
+	g_return_if_fail (PK_IS_PACKAGE (package));
+	g_free (package->priv->summary);
+	package->priv->summary = g_strdup (summary);
+}
+
 /**
  * pk_package_get_id:
  * @package: a valid #PkPackage instance
@@ -461,11 +500,10 @@ pk_package_set_property (GObject *object, guint prop_id, const GValue *value, GP
 
 	switch (prop_id) {
 	case PROP_INFO:
-		priv->info = g_value_get_uint (value);
+		pk_package_set_info (package, g_value_get_uint (value));
 		break;
 	case PROP_SUMMARY:
-		g_free (priv->summary);
-		priv->summary = g_strdup (g_value_get_string (value));
+		pk_package_set_summary (package, g_value_get_string (value));
 		break;
 	case PROP_LICENSE:
 		g_free (priv->license);
@@ -779,6 +817,10 @@ static void
 pk_package_init (PkPackage *package)
 {
 	package->priv = PK_PACKAGE_GET_PRIVATE (package);
+	package->priv->package_id_split[PK_PACKAGE_ID_NAME] = NULL;
+	package->priv->package_id_split[PK_PACKAGE_ID_VERSION] = NULL;
+	package->priv->package_id_split[PK_PACKAGE_ID_ARCH] = NULL;
+	package->priv->package_id_split[PK_PACKAGE_ID_DATA] = NULL;
 }
 
 /**
@@ -805,7 +847,7 @@ pk_package_finalize (GObject *object)
 	g_free (priv->update_changelog);
 	g_free (priv->update_issued);
 	g_free (priv->update_updated);
-	g_strfreev (priv->package_id_split);
+	g_free (priv->package_id_data);
 
 	G_OBJECT_CLASS (pk_package_parent_class)->finalize (object);
 }
