@@ -53,7 +53,6 @@ struct _PkResultsPrivate
 	PkProgress		*progress;
 	PkExitEnum		 exit_enum;
 	PkError			*error_code;
-	GPtrArray		*package_array;
 	GPtrArray		*details_array;
 	GPtrArray		*update_detail_array;
 	GPtrArray		*category_array;
@@ -174,24 +173,15 @@ pk_results_set_exit_code (PkResults *results, PkExitEnum exit_enum)
 gboolean
 pk_results_add_package (PkResults *results, PkPackage *item)
 {
-	PkInfoEnum info;
-
 	g_return_val_if_fail (PK_IS_RESULTS (results), FALSE);
 	g_return_val_if_fail (item != NULL, FALSE);
 
 	/* do not allow finished types */
-	g_object_get (item,
-		      "info", &info,
-		      NULL);
-	if (info == PK_INFO_ENUM_FINISHED) {
-		g_warning ("internal error: finished packages cannot be added to a PkResults object");
+	if (pk_package_get_info (item) == PK_INFO_ENUM_FINISHED) {
+		g_warning ("Finished packages cannot be added to PkResults");
 		return FALSE;
 	}
-
-	/* copy and add to array */
 	pk_package_sack_add_package (results->priv->package_sack, item);
-	g_ptr_array_add (results->priv->package_array, g_object_ref (item));
-
 	return TRUE;
 }
 
@@ -590,7 +580,7 @@ GPtrArray *
 pk_results_get_package_array (PkResults *results)
 {
 	g_return_val_if_fail (PK_IS_RESULTS (results), NULL);
-	return g_ptr_array_ref (results->priv->package_array);
+	return pk_package_sack_get_array (results->priv->package_sack);
 }
 
 /**
@@ -922,7 +912,6 @@ pk_results_init (PkResults *results)
 	results->priv->inputs = 0;
 	results->priv->progress = NULL;
 	results->priv->error_code = NULL;
-	results->priv->package_array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	results->priv->package_sack = pk_package_sack_new ();
 	results->priv->details_array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	results->priv->update_detail_array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
@@ -947,7 +936,6 @@ pk_results_finalize (GObject *object)
 	PkResults *results = PK_RESULTS (object);
 	PkResultsPrivate *priv = results->priv;
 
-	g_ptr_array_unref (priv->package_array);
 	g_ptr_array_unref (priv->details_array);
 	g_ptr_array_unref (priv->update_detail_array);
 	g_ptr_array_unref (priv->category_array);
