@@ -22,6 +22,7 @@
 #include "config.h"
 
 #include <stdio.h>
+#include <string.h>
 #include <gio/gio.h>
 #include <glib.h>
 #include <packagekit-glib2/pk-results.h>
@@ -33,6 +34,7 @@
 /* tiny helper to help us do the async operation */
 typedef struct {
 	GError		**error;
+	GMainContext	*context;
 	GMainLoop	*loop;
 	PkResults	*results;
 } PkTaskHelper;
@@ -79,28 +81,33 @@ PkResults *
 pk_task_remove_packages_sync (PkTask *task, gchar **package_ids, gboolean allow_deps, gboolean autoremove, GCancellable *cancellable,
 			      PkProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_remove_packages_async (task, package_ids, allow_deps, autoremove, cancellable, progress_callback, progress_user_data,
-				       (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				       (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -127,28 +134,33 @@ PkResults *
 pk_task_install_packages_sync (PkTask *task, gchar **package_ids, GCancellable *cancellable,
 			       PkProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_install_packages_async (task, package_ids, cancellable, progress_callback, progress_user_data,
-					(GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+					(GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -175,28 +187,33 @@ PkResults *
 pk_task_update_packages_sync (PkTask *task, gchar **package_ids, GCancellable *cancellable,
 			      PkProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_update_packages_async (task, package_ids, cancellable, progress_callback, progress_user_data,
-				       (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				       (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -224,28 +241,33 @@ PkResults *
 pk_task_install_files_sync (PkTask *task, gchar **files, GCancellable *cancellable,
 			    PkProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_install_files_async (task, files, cancellable, progress_callback, progress_user_data,
-				     (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				     (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -271,28 +293,33 @@ pk_task_resolve_sync (PkTask *task, PkBitfield filters, gchar **packages, GCance
 		      PkProgressCallback progress_callback, gpointer progress_user_data,
 		      GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_resolve_async (task, filters, packages, cancellable, progress_callback, progress_user_data,
-			       (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+			       (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -318,28 +345,33 @@ pk_task_search_names_sync (PkTask *task, PkBitfield filters, gchar **values, GCa
 			   PkProgressCallback progress_callback, gpointer progress_user_data,
 			   GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_search_names_async (task, filters, values, cancellable, progress_callback, progress_user_data,
-				    (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				    (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -365,28 +397,33 @@ pk_task_search_details_sync (PkTask *task, PkBitfield filters, gchar **values, G
 			     PkProgressCallback progress_callback, gpointer progress_user_data,
 			     GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_search_details_async (task, filters, values, cancellable, progress_callback, progress_user_data,
-				      (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				      (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -412,28 +449,33 @@ pk_task_search_groups_sync (PkTask *task, PkBitfield filters, gchar **values, GC
 			    PkProgressCallback progress_callback, gpointer progress_user_data,
 			    GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_search_groups_async (task, filters, values, cancellable, progress_callback, progress_user_data,
-				     (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				     (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -459,28 +501,33 @@ pk_task_search_files_sync (PkTask *task, PkBitfield filters, gchar **values, GCa
 			   PkProgressCallback progress_callback, gpointer progress_user_data,
 			   GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_search_files_async (task, filters, values, cancellable, progress_callback, progress_user_data,
-				    (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				    (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -505,28 +552,33 @@ pk_task_get_details_sync (PkTask *task, gchar **package_ids, GCancellable *cance
 			  PkProgressCallback progress_callback, gpointer progress_user_data,
 			  GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_get_details_async (task, package_ids, cancellable, progress_callback, progress_user_data,
-				   (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				   (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -551,28 +603,33 @@ pk_task_get_update_detail_sync (PkTask *task, gchar **package_ids, GCancellable
 				PkProgressCallback progress_callback, gpointer progress_user_data,
 				GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_get_update_detail_async (task, package_ids, cancellable, progress_callback, progress_user_data,
-				         (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				         (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -598,28 +655,33 @@ pk_task_download_packages_sync (PkTask *task, gchar **package_ids, const gchar *
 				PkProgressCallback progress_callback, gpointer progress_user_data,
 				GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_download_packages_async (task, package_ids, directory, cancellable, progress_callback, progress_user_data,
-				         (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				         (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -644,28 +706,33 @@ pk_task_get_updates_sync (PkTask *task, PkBitfield filters, GCancellable *cancel
 			  PkProgressCallback progress_callback, gpointer progress_user_data,
 			  GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_get_updates_async (task, filters, cancellable, progress_callback, progress_user_data,
-				     (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				     (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -692,28 +759,34 @@ pk_task_get_depends_sync (PkTask *task, PkBitfield filters, gchar **package_ids,
 			  PkProgressCallback progress_callback, gpointer progress_user_data,
 			  GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_get_depends_async (task, filters, package_ids, recursive, cancellable, progress_callback, progress_user_data,
-				     (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				     (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
 
-	results = helper->results;
+	g_main_loop_run (helper.loop);
+
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -738,28 +811,33 @@ pk_task_get_packages_sync (PkTask *task, PkBitfield filters, GCancellable *cance
 			   PkProgressCallback progress_callback, gpointer progress_user_data,
 			   GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_get_packages_async (task, filters, cancellable, progress_callback, progress_user_data,
-				    (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				    (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -786,28 +864,34 @@ pk_task_get_requires_sync (PkTask *task, PkBitfield filters, gchar **package_ids
 			   PkProgressCallback progress_callback, gpointer progress_user_data,
 			   GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_get_requires_async (task, filters, package_ids, recursive, cancellable, progress_callback, progress_user_data,
-				    (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				    (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
 
-	results = helper->results;
+	g_main_loop_run (helper.loop);
+
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -834,28 +918,34 @@ pk_task_what_provides_sync (PkTask *task, PkBitfield filters, PkProvidesEnum pro
 			    PkProgressCallback progress_callback, gpointer progress_user_data,
 			    GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_what_provides_async (task, filters, provides, values, cancellable, progress_callback, progress_user_data,
-				     (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				     (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
 
-	results = helper->results;
+	g_main_loop_run (helper.loop);
+
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -880,28 +970,33 @@ pk_task_get_files_sync (PkTask *task, gchar **package_ids, GCancellable *cancell
 			PkProgressCallback progress_callback, gpointer progress_user_data,
 			GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_get_files_async (task, package_ids, cancellable, progress_callback, progress_user_data,
-				 (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				 (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -925,28 +1020,33 @@ pk_task_get_categories_sync (PkTask *task, GCancellable *cancellable,
 			     PkProgressCallback progress_callback, gpointer progress_user_data,
 			     GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_get_categories_async (task, cancellable, progress_callback, progress_user_data,
-				      (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				      (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -971,28 +1071,33 @@ pk_task_refresh_cache_sync (PkTask *task, gboolean force, GCancellable *cancella
 			    PkProgressCallback progress_callback, gpointer progress_user_data,
 			    GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_refresh_cache_async (task, force, cancellable, progress_callback, progress_user_data,
-				     (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				     (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -1017,28 +1122,33 @@ pk_task_get_repo_list_sync (PkTask *task, PkBitfield filters, GCancellable *canc
 			    PkProgressCallback progress_callback, gpointer progress_user_data,
 			    GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_get_repo_list_async (task, filters, cancellable, progress_callback, progress_user_data,
-				     (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				     (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -1064,28 +1174,33 @@ pk_task_repo_enable_sync (PkTask *task, const gchar *repo_id, gboolean enabled,
 			  PkProgressCallback progress_callback, gpointer progress_user_data,
 			  GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_repo_enable_async (task, repo_id, enabled, cancellable, progress_callback, progress_user_data,
-				   (GAsyncReadyCallback) pk_task_generic_finish_sync, helper);
+				   (GAsyncReadyCallback) pk_task_generic_finish_sync, &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
@@ -1113,29 +1228,34 @@ pk_task_repair_system_sync (PkTask *task, GCancellable *cancellable,
                             PkProgressCallback progress_callback, gpointer progress_user_data,
                             GError **error)
 {
-	PkTaskHelper *helper;
+	PkTaskHelper helper;
 	PkResults *results;
 
 	g_return_val_if_fail (PK_IS_TASK (task), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* create temp object */
-	helper = g_new0 (PkTaskHelper, 1);
-	helper->loop = g_main_loop_new (NULL, FALSE);
-	helper->error = error;
+	memset (&helper, 0, sizeof (PkTaskHelper));
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
 
 	/* run async method */
 	pk_task_repair_system_async (task, cancellable, progress_callback, progress_user_data,
-	                             (GAsyncReadyCallback) pk_task_generic_finish_sync,
-	                             helper);
+				     (GAsyncReadyCallback) pk_task_generic_finish_sync,
+				     &helper);
 
-	g_main_loop_run (helper->loop);
+	g_main_loop_run (helper.loop);
 
-	results = helper->results;
+	results = helper.results;
+
+	g_main_context_pop_thread_default (helper.context);
 
 	/* free temp object */
-	g_main_loop_unref (helper->loop);
-	g_free (helper);
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
 
 	return results;
 }
