@@ -19,11 +19,12 @@
 #
 # This file contain the base classes to implement a PackageKit python backend
 #
-from __future__ import print_function
+
 
 # imports
+from __future__ import print_function
+
 import sys
-import codecs
 import traceback
 import os.path
 
@@ -33,53 +34,17 @@ PACKAGE_IDS_DELIM = '&'
 FILENAME_DELIM = '|'
 
 def _to_unicode(txt, encoding='utf-8'):
-    if isinstance(txt, basestring):
-        if not isinstance(txt, unicode):
-            txt = unicode(txt, encoding, errors='replace')
+    if isinstance(txt, str):
+        if not isinstance(txt, str):
+            txt = str(txt, encoding, errors='replace')
     return txt
 
 def _to_utf8(txt, errors='replace'):
-    '''convert practically anything to a utf-8-encoded byte string'''
-
-    # convert to unicode object
     if isinstance(txt, str):
-        txt = txt.decode('utf-8', errors=errors)
-    if not isinstance(txt, basestring):
-        # try to convert non-string objects like exceptions
-        try:
-            # if txt.__unicode__() exists, or txt.__str__() returns ASCII
-            txt = unicode(txt)
-        except UnicodeDecodeError:
-            # if txt.__str__() exists
-            txt = str(txt).decode('utf-8', errors=errors)
-        except:
-            # no __str__(), __unicode__() methods, use representation
-            txt = unicode(repr(txt))
-
-    # return encoded as UTF-8
-    return txt.encode('utf-8', errors=errors)
-
-# Classes
-
-class _UTF8Writer(codecs.StreamWriter):
-
-    encoding = 'utf-8'
-
-    def __init__(self, stream, errors='replace'):
-        codecs.StreamWriter.__init__(self, stream, errors)
-
-    def encode(self, inp, errors='strict'):
-        try:
-            l = len(inp)
-        except TypeError:
-            try:
-                l = len(unicode(inp))
-            except:
-                try:
-                    l = len(str(inp))
-                except:
-                    l = 1
-        return (_to_utf8(inp, errors=errors), l)
+        return txt
+    if isinstance(txt, unicode):
+        return txt.encode('utf-8', errors=errors)
+    return str(txt)
 
 class PkError(Exception):
     def __init__(self, code, details):
@@ -91,10 +56,6 @@ class PkError(Exception):
 class PackageKitBaseBackend:
 
     def __init__(self, cmds):
-        # Make sys.stdout/stderr cope with UTF-8
-        sys.stdout = _UTF8Writer(sys.stdout)
-        sys.stderr = _UTF8Writer(sys.stderr)
-
         # Setup a custom exception handler
         installExceptionHandler(self)
         self.cmds = cmds
@@ -105,7 +66,6 @@ class PackageKitBaseBackend:
         self.interactive = False
         self.cache_age = 0
         self.percentage_old = 0
-        self.sub_percentage_old = 0
 
         # try to get LANG
         try:
@@ -157,9 +117,9 @@ class PackageKitBaseBackend:
         @param percent: Progress percentage (int preferred)
         '''
         if percent == None:
-            print("no-percentage-updates")
+            sys.stdout.write(_to_utf8("no-percentage-updates\n"))
         elif percent == 0 or percent > self.percentage_old:
-            print("percentage\t%i" % (percent))
+            sys.stdout.write(_to_utf8("percentage\t%i\n" % percent))
             self.percentage_old = percent
         sys.stdout.flush()
 
@@ -168,26 +128,16 @@ class PackageKitBaseBackend:
         Write progress speed
         @param bps: Progress speed (int, bytes per second)
         '''
-        print("speed\t%i" % (bps))
+        sys.stdout.write(_to_utf8("speed\t%i\n" % bps))
         sys.stdout.flush()
 
-    def item_percentage(self, package_id, percent=None):
+    def item_progress(self, package_id, status, percent=None):
         '''
         send 'itemprogress' signal
         @param package_id: The package ID name, e.g. openoffice-clipart;2.6.22;ppc64;fedora
         @param percent: percentage of the current item (int preferred)
         '''
-        print("item-percentage\t%s\t%i" % (package_id, percent))
-        sys.stdout.flush()
-
-    def sub_percentage(self, percent=None):
-        '''
-        send 'subpercentage' signal : subprogress percentage
-        @param percent: subprogress percentage (int preferred)
-        '''
-        if percent == 0 or percent > self.sub_percentage_old:
-            print("subpercentage\t%i" % (percent))
-            self.sub_percentage_old = percent
+        sys.stdout.write(_to_utf8("item-progress\t%s\t%s\t%i\n" % (package_id, status, percent)))
         sys.stdout.flush()
 
     def error(self, err, description, exit=True):
@@ -202,7 +152,7 @@ class PackageKitBaseBackend:
             self.unLock()
 
         # this should be fast now
-        print("error\t%s\t%s" % (err, description))
+        sys.stdout.write(_to_utf8("error\t%s\t%s\n" % (err, description)))
         sys.stdout.flush()
         if exit:
             # Paradoxically, we don't want to print "finished" to stdout here.
@@ -210,14 +160,14 @@ class PackageKitBaseBackend:
             # huge race when you try to change a dispatcher because of an error.
             #
             # Leave PackageKit to clean up for us in this case.
-            sys.exit(1)
+            sys.exit(254)
 
     def message(self, typ, msg):
         '''
         send 'message' signal
         @param typ: MESSAGE_BROKEN_MIRROR
         '''
-        print("message\t%s\t%s" % (typ, msg))
+        sys.stdout.write(_to_utf8("message\t%s\t%s\n" % (typ, msg)))
         sys.stdout.flush()
 
     def package(self, package_id, status, summary):
@@ -227,7 +177,7 @@ class PackageKitBaseBackend:
         @param package_id: The package ID name, e.g. openoffice-clipart;2.6.22;ppc64;fedora
         @param summary: The package Summary
         '''
-        print("package\t%s\t%s\t%s" % (status, package_id, summary), file=sys.stdout)
+        sys.stdout.write(_to_utf8("package\t%s\t%s\t%s\n" % (status, package_id, summary)))
         sys.stdout.flush()
 
     def media_change_required(self, mtype, id, text):
@@ -237,7 +187,7 @@ class PackageKitBaseBackend:
         @param id: the localised label of the media
         @param text: the localised text describing the media
         '''
-        print("media-change-required\t%s\t%s\t%s" % (mtype, id, text), file=sys.stdout)
+        sys.stdout.write(_to_utf8("media-change-required\t%s\t%s\t%s\n" % (mtype, id, text)))
         sys.stdout.flush()
 
     def distro_upgrade(self, dtype, name, summary):
@@ -247,7 +197,7 @@ class PackageKitBaseBackend:
         @param name: The distro name, e.g. "fedora-9"
         @param summary: The localised distribution name and description
         '''
-        print("distro-upgrade\t%s\t%s\t%s" % (dtype, name, summary), file=sys.stdout)
+        sys.stdout.write(_to_utf8("distro-upgrade\t%s\t%s\t%s\n" % (dtype, name, summary)))
         sys.stdout.flush()
 
     def status(self, state):
@@ -255,7 +205,7 @@ class PackageKitBaseBackend:
         send 'status' signal
         @param state: STATUS_DOWNLOAD, STATUS_INSTALL, STATUS_UPDATE, STATUS_REMOVE, STATUS_WAIT
         '''
-        print("status\t%s" % (state))
+        sys.stdout.write(_to_utf8("status\t%s\n" % state))
         sys.stdout.flush()
 
     def repo_detail(self, repoid, name, state):
@@ -264,7 +214,7 @@ class PackageKitBaseBackend:
         @param repoid: The repo id tag
         @param state: false is repo is disabled else true.
         '''
-        print("repo-detail\t%s\t%s\t%s" % (repoid, name, _bool_to_string(state)), file=sys.stdout)
+        sys.stdout.write(_to_utf8("repo-detail\t%s\t%s\t%s\n" % (repoid, name, _bool_to_string(state))))
         sys.stdout.flush()
 
     def data(self, data):
@@ -272,7 +222,7 @@ class PackageKitBaseBackend:
         send 'data' signal:
         @param data:  The current worked on package
         '''
-        print("data\t%s" % (data))
+        sys.stdout.write(_to_utf8("data\t%s\n" % data))
         sys.stdout.flush()
 
     def details(self, package_id, package_license, group, desc, url, bytes):
@@ -285,7 +235,7 @@ class PackageKitBaseBackend:
         @param url: The upstream project homepage
         @param bytes: The size of the package, in bytes
         '''
-        print("details\t%s\t%s\t%s\t%s\t%s\t%ld" % (package_id, package_license, group, desc, url, bytes), file=sys.stdout)
+        sys.stdout.write(_to_utf8("details\t%s\t%s\t%s\t%s\t%s\t%ld\n" % (package_id, package_license, group, desc, url, bytes)))
         sys.stdout.flush()
 
     def files(self, package_id, file_list):
@@ -293,7 +243,7 @@ class PackageKitBaseBackend:
         Send 'files' signal
         @param file_list: List of the files in the package, separated by ';'
         '''
-        print("files\t%s\t%s" % (package_id, file_list), file=sys.stdout)
+        sys.stdout.write(_to_utf8("files\t%s\t%s\n" % (package_id, file_list)))
         sys.stdout.flush()
 
     def category(self, parent_id, cat_id, name, summary, icon):
@@ -305,14 +255,14 @@ class PackageKitBaseBackend:
         summery   : a summary of the category in current locale.
         icon      : an icon name to represent the category
         '''
-        print("category\t%s\t%s\t%s\t%s\t%s" % (parent_id, cat_id, name, summary, icon), file=sys.stdout)
+        sys.stdout.write(_to_utf8("category\t%s\t%s\t%s\t%s\t%s\n" % (parent_id, cat_id, name, summary, icon)))
         sys.stdout.flush()
 
     def finished(self):
         '''
         Send 'finished' signal
         '''
-        print("finished", file=sys.stdout)
+        sys.stdout.write(_to_utf8("finished\n"))
         sys.stdout.flush()
 
     def update_detail(self, package_id, updates, obsoletes, vendor_url, bugzilla_url, cve_url, restart, update_text, changelog, state, issued, updated):
@@ -331,7 +281,7 @@ class PackageKitBaseBackend:
         @param issued:
         @param updated:
         '''
-        print("updatedetail\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" % (package_id, updates, obsoletes, vendor_url, bugzilla_url, cve_url, restart, update_text, changelog, state, issued, updated), file=sys.stdout)
+        sys.stdout.write(_to_utf8("updatedetail\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" % (package_id, updates, obsoletes, vendor_url, bugzilla_url, cve_url, restart, update_text, changelog, state, issued, updated)))
         sys.stdout.flush()
 
     def require_restart(self, restart_type, details):
@@ -340,7 +290,7 @@ class PackageKitBaseBackend:
         @param restart_type: RESTART_SYSTEM, RESTART_APPLICATION, RESTART_SESSION
         @param details: Optional details about the restart
         '''
-        print("requirerestart\t%s\t%s" % (restart_type, details))
+        sys.stdout.write(_to_utf8("requirerestart\t%s\t%s\n" % (restart_type, details)))
         sys.stdout.flush()
 
     def allow_cancel(self, allow):
@@ -352,7 +302,7 @@ class PackageKitBaseBackend:
             data = 'true'
         else:
             data = 'false'
-        print("allow-cancel\t%s" % (data))
+        sys.stdout.write(_to_utf8("allow-cancel\t%s\n" % data))
         sys.stdout.flush()
 
     def repo_signature_required(self, package_id, repo_name, key_url, key_userid, key_id, key_fingerprint, key_timestamp, sig_type):
@@ -367,9 +317,9 @@ class PackageKitBaseBackend:
         @param key_timestamp:   Key timestamp
         @param sig_type:        Key type (GPG)
         '''
-        print("repo-signature-required\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" % (
+        sys.stdout.write(_to_utf8("repo-signature-required\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" % (
             package_id, repo_name, key_url, key_userid, key_id, key_fingerprint, key_timestamp, sig_type
-            ))
+            )))
         sys.stdout.flush()
 
     def eula_required(self, eula_id, package_id, vendor_name, license_agreement):
@@ -380,9 +330,9 @@ class PackageKitBaseBackend:
         @param vendor_name:     Name of the vendor that wrote the EULA
         @param license_agreement: The license text
         '''
-        print("eula-required\t%s\t%s\t%s\t%s" % (
+        sys.stdout.write(_to_utf8("eula-required\t%s\t%s\t%s\t%s\n" % (
             eula_id, package_id, vendor_name, license_agreement
-            ))
+            )))
         sys.stdout.flush()
 
 #
@@ -452,13 +402,6 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def update_system(self, only_trusted):
-        '''
-        Implement the {backend}-update-system functionality
-        Needed to be implemented in a sub class
-        '''
-        self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
-
     def upgrade_system(self, distro_id):
         '''
         Implement the {backend}-update-system functionality
@@ -473,7 +416,7 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def install_packages(self, only_trusted, package_ids):
+    def install_packages(self, transaction_flags, package_ids):
         '''
         Implement the {backend}-install functionality
         Needed to be implemented in a sub class
@@ -487,7 +430,7 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def install_files(self, only_trusted, inst_files):
+    def install_files(self, transaction_flags, inst_files):
         '''
         Implement the {backend}-install_files functionality
         Install the package containing the inst_file file
@@ -502,14 +445,14 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def remove_packages(self, allowdep, autoremove, package_ids):
+    def remove_packages(self, transaction_flags, allowdep, autoremove, package_ids):
         '''
         Implement the {backend}-remove functionality
         Needed to be implemented in a sub class
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def update_packages(self, only_trusted, package_ids):
+    def update_packages(self, transaction_flags, package_ids):
         '''
         Implement the {backend}-update functionality
         Needed to be implemented in a sub class
@@ -593,35 +536,7 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def simulate_install_files (self, inst_files):
-        '''
-        Implement the {backend}-simulate-install-files functionality
-        Needed to be implemented in a sub class
-        '''
-        self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
-
-    def simulate_install_packages(self, package_ids):
-        '''
-        Implement the {backend}-simulate-install-packages functionality
-        Needed to be implemented in a sub class
-        '''
-        self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
-
-    def simulate_remove_packages(self, package_ids):
-        '''
-        Implement the {backend}-simulate-remove-packages functionality
-        Needed to be implemented in a sub class
-        '''
-        self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
-
-    def simulate_update_packages(self, package_ids):
-        '''
-        Implement the {backend}-simulate-update-packages functionality
-        Needed to be implemented in a sub class
-        '''
-        self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
-
-    def repair_system(self, only_trusted):
+    def repair_system(self, transaction_flags):
         '''
         Implement the {backend}-repair-system functionality
         Needed to be implemented in a sub class
@@ -629,14 +544,6 @@ class PackageKitBaseBackend:
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend",
                    exit=False)
 
-    def simulate_repair_system(self):
-        '''
-        Implement the {backend}-simulate-repair-system functionality
-        Needed to be implemented in a sub class
-        '''
-        self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend",
-                   exit=False)
-
     def customTracebackHandler(self, tb):
         '''
         Custom Traceback Handler
@@ -703,14 +610,14 @@ class PackageKitBaseBackend:
             self.get_updates(filters)
             self.finished()
         elif cmd == 'install-files':
-            only_trusted = _text_to_bool(args[0])
+            transaction_flags = args[0].split(';')
             files_to_inst = args[1].split(FILENAME_DELIM)
-            self.install_files(only_trusted, files_to_inst)
+            self.install_files(transaction_flags, files_to_inst)
             self.finished()
         elif cmd == 'install-packages':
-            only_trusted = _text_to_bool(args[0])
+            transaction_flags = args[0].split(';')
             package_ids = args[1].split(PACKAGE_IDS_DELIM)
-            self.install_packages(only_trusted, package_ids)
+            self.install_packages(transaction_flags, package_ids)
             self.finished()
         elif cmd == 'install-signature':
             sigtype = args[0]
@@ -723,10 +630,11 @@ class PackageKitBaseBackend:
             self.refresh_cache(force)
             self.finished()
         elif cmd == 'remove-packages':
-            allowdeps = _text_to_bool(args[0])
-            autoremove = _text_to_bool(args[1])
-            package_ids = args[2].split(PACKAGE_IDS_DELIM)
-            self.remove_packages(allowdeps, autoremove, package_ids)
+            transaction_flags = args[0].split(';')
+            package_ids = args[1].split(PACKAGE_IDS_DELIM)
+            allowdeps = _text_to_bool(args[2])
+            autoremove = _text_to_bool(args[3])
+            self.remove_packages(transaction_flags, package_ids, allowdeps, autoremove)
             self.finished()
         elif cmd == 'repo-enable':
             repoid = args[0]
@@ -769,13 +677,9 @@ class PackageKitBaseBackend:
             self.repo_signature_install(package)
             self.finished()
         elif cmd == 'update-packages':
-            only_trusted = _text_to_bool(args[0])
+            transaction_flags = args[0].split(';')
             package_ids = args[1].split(PACKAGE_IDS_DELIM)
-            self.update_packages(only_trusted, package_ids)
-            self.finished()
-        elif cmd == 'update-system':
-            only_trusted = _text_to_bool(args[0])
-            self.update_system(only_trusted)
+            self.update_packages(transaction_flags, package_ids)
             self.finished()
         elif cmd == 'what-provides':
             filters = args[0].split(';')
@@ -790,31 +694,12 @@ class PackageKitBaseBackend:
         elif cmd == 'get-categories':
             self.get_categories()
             self.finished()
-        elif cmd == 'simulate-install-files':
-            files_to_inst = args[0].split(FILENAME_DELIM)
-            self.simulate_install_files(files_to_inst)
-            self.finished()
-        elif cmd == 'simulate-install-packages':
-            package_ids = args[0].split(PACKAGE_IDS_DELIM)
-            self.simulate_install_packages(package_ids)
-            self.finished()
-        elif cmd == 'simulate-remove-packages':
-            package_ids = args[0].split(PACKAGE_IDS_DELIM)
-            self.simulate_remove_packages(package_ids)
-            self.finished()
-        elif cmd == 'simulate-update-packages':
-            package_ids = args[0].split(PACKAGE_IDS_DELIM)
-            self.simulate_update_packages(package_ids)
-            self.finished()
         elif cmd == 'upgrade-system':
             self.upgrade_system(args[0])
             self.finished()
         elif cmd == 'repair-system':
             self.repair_system(args[0])
             self.finished()
-        elif cmd == 'simulate-repair-system':
-            self.simulate_repair_system()
-            self.finished()
         else:
             errmsg = "command '%s' is not known" % cmd
             self.error(ERROR_INTERNAL_ERROR, errmsg, exit=False)
@@ -845,8 +730,8 @@ def format_string(text, encoding='utf-8'):
     '''
     Format a string to be used on stdout for communication with the daemon.
     '''
-    if not isinstance(text, unicode):
-        text = unicode(text, encoding, errors='replace')
+    if not isinstance(text, str):
+        text = str(text, encoding, errors='replace')
     return text.replace("\n", ";")
 
 def _text_to_bool(text):
