@@ -129,7 +129,7 @@ void		 pk_backend_job_set_cmdline		(PkBackendJob	*job,
 const gchar	*pk_backend_job_get_cmdline		(PkBackendJob	*job);
 void		 pk_backend_job_set_locale		(PkBackendJob	*job,
 							 const gchar	*code);
-gboolean	 pk_backend_job_set_frontend_socket	(PkBackendJob	*job,
+void		 pk_backend_job_set_frontend_socket	(PkBackendJob	*job,
 							 const gchar	*frontend_socket);
 void		 pk_backend_job_set_cache_age		(PkBackendJob	*job,
 							 guint		 cache_age);
@@ -190,6 +190,7 @@ void		 pk_backend_job_update_detail		(PkBackendJob	*job,
 void		 pk_backend_job_require_restart		(PkBackendJob	*job,
 							 PkRestartEnum	 restart,
 							 const gchar	*package_id);
+G_DEPRECATED
 void		 pk_backend_job_message			(PkBackendJob	*job,
 							 PkMessageEnum	 message,
 							 const gchar	*details, ...)
