@@ -28,13 +28,10 @@
 #include <dbus/dbus-glib.h>
 
 #include "pk-network-stack-connman.h"
-#include "pk-conf.h"
-#include "pk-marshal.h"
 
 struct PkNetworkStackConnmanPrivate
 {
 	guint			 watch_id;
-	PkConf			*conf;
 	gboolean		 is_enabled;
 	GDBusConnection		*bus;
 	GDBusProxy		*proxy;
@@ -257,11 +254,8 @@ pk_network_stack_connman_appeared_cb (GDBusConnection *connection,
 				      const gchar *name_owner,
 				      gpointer user_data)
 {
-	gboolean ret;
 	PkNetworkStackConnman *nstack_connman = PK_NETWORK_STACK_CONNMAN (user_data);
-	ret = pk_conf_get_bool (nstack_connman->priv->conf,
-				"UseNetworkConnman");
-	nstack_connman->priv->is_enabled = ret;
+	nstack_connman->priv->is_enabled = TRUE;
 }
 
 /**
@@ -276,6 +270,38 @@ pk_network_stack_connman_vanished_cb (GDBusConnection *connection,
 	nstack_connman->priv->is_enabled = FALSE;
 }
 
+static void
+pk_marshal_VOID__STRING_BOXED (GClosure *closure,
+			       GValue *return_value G_GNUC_UNUSED,
+			       guint n_param_values,
+			       const GValue *param_values,
+			       gpointer invocation_hint G_GNUC_UNUSED,
+			       gpointer marshal_data)
+{
+	typedef void (*GMarshalFunc_VOID__STRING_BOXED) (gpointer data1,
+		gpointer arg_1,
+		gpointer arg_2,
+		gpointer data2);
+		register GMarshalFunc_VOID__STRING_BOXED callback;
+		register GCClosure *cc = (GCClosure*) closure;
+		register gpointer data1, data2;
+
+	g_return_if_fail (n_param_values == 3);
+
+	if (G_CCLOSURE_SWAP_DATA (closure)) {
+		data1 = closure->data;
+		data2 = g_value_get_pointer (param_values + 0);
+	} else {
+		data1 = g_value_get_pointer (param_values + 0);
+		data2 = closure->data;
+	}
+	callback = (GMarshalFunc_VOID__STRING_BOXED) (marshal_data ? marshal_data : cc->callback);
+	callback (data1,
+		 g_value_get_string (param_values + 1),
+		 g_value_get_boxed (param_values + 2),
+		 data2);
+}
+
 /**
  * pk_network_stack_connman_init:
  **/
@@ -286,7 +312,6 @@ pk_network_stack_connman_init (PkNetworkStackConnman *nstack_connman)
 	GDBusProxy *proxy;
 
 	nstack_connman->priv = PK_NETWORK_STACK_CONNMAN_GET_PRIVATE (nstack_connman);
-	nstack_connman->priv->conf = pk_conf_new ();
 
 	dbus_g_object_register_marshaller (pk_marshal_VOID__STRING_BOXED,
 					   G_TYPE_NONE, G_TYPE_STRING,
@@ -345,7 +370,6 @@ pk_network_stack_connman_finalize (GObject *object)
 	g_return_if_fail (nstack_connman->priv != NULL);
 
 	g_bus_unwatch_name (nstack_connman->priv->watch_id);
-	g_object_unref (nstack_connman->priv->conf);
 	if (nstack_connman->priv->proxy != NULL)
 		g_object_unref (nstack_connman->priv->proxy);
 
