@@ -63,6 +63,7 @@
 #include <packagekit-glib2/pk-common.h>
 
 #include "pk-conf.h"
+#include "pk-shared.h"
 #include "pk-transaction.h"
 #include "pk-transaction-private.h"
 #include "pk-transaction-list.h"
@@ -74,6 +75,12 @@ static void     pk_transaction_list_finalize	(GObject	*object);
 /* the interval between each CST, in seconds */
 #define PK_TRANSACTION_WEDGE_CHECK			10
 
+/* How long the transaction should be queriable after it is finished */
+#define PK_TRANSACTION_KEEP_FINISHED_TIMOUT		5 /* s */
+
+/* how many times we should retry a locked transaction */
+#define PK_TRANSACTION_LIST_MAX_LOCK_RETRIES	4
+
 struct PkTransactionListPrivate
 {
 	GPtrArray		*array;
@@ -82,6 +89,7 @@ struct PkTransactionListPrivate
 	PkConf			*conf;
 	GPtrArray		*plugins;
 	PkBackend		*backend;
+	GDBusNodeInfo		*introspection;
 };
 
 typedef struct {
@@ -213,7 +221,6 @@ pk_transaction_list_remove_internal (PkTransactionList *tlist, PkTransactionItem
 	g_return_val_if_fail (item != NULL, FALSE);
 
 	/* valid item */
-	g_debug ("remove transaction %s, item %p", item->tid, item);
 	ret = g_ptr_array_remove (tlist->priv->array, item);
 	if (!ret) {
 		g_warning ("could not remove %p as not present in list", item);
@@ -270,26 +277,29 @@ pk_transaction_list_remove (PkTransactionList *tlist, const gchar *tid)
 /**
  * pk_transaction_list_set_background:
  **/
-gboolean
-pk_transaction_list_set_background (PkTransactionList *tlist, const gchar *tid, gboolean background)
+void
+pk_transaction_list_set_background (PkTransactionList *tlist,
+				    const gchar *tid,
+				    gboolean background)
 {
 	PkTransactionItem *item;
 
-	g_return_val_if_fail (PK_IS_TRANSACTION_LIST (tlist), FALSE);
-	g_return_val_if_fail (tid != NULL, FALSE);
+	g_return_if_fail (PK_IS_TRANSACTION_LIST (tlist));
+	g_return_if_fail (tid != NULL);
 
 	item = pk_transaction_list_get_from_tid (tlist, tid);
 	if (item == NULL) {
-		g_warning ("could not get item");
-		return FALSE;
+		g_warning ("could not get %s", tid);
+		return;
 	}
+	if (item->background == background)
+		return;
 	if (pk_transaction_get_state (item->transaction) == PK_TRANSACTION_STATE_FINISHED) {
-		g_debug ("already finished, so waiting to timeout");
-		return FALSE;
+		g_debug ("already finished %s, so waiting to timeout", tid);
+		return;
 	}
 	g_debug ("%s is now background: %i", tid, background);
 	item->background = background;
-	return TRUE;
 }
 
 /**
@@ -312,7 +322,6 @@ pk_transaction_list_run_idle_cb (PkTransactionItem *item)
 	gboolean ret;
 
 	/* run the transaction */
-	g_debug ("actually running %s", item->tid);
 	pk_transaction_set_backend (item->transaction,
 				    item->list->priv->backend);
 	ret = pk_transaction_run (item->transaction);
@@ -331,7 +340,6 @@ static void
 pk_transaction_list_run_item (PkTransactionList *tlist, PkTransactionItem *item)
 {
 	/* we set this here so that we don't try starting more than one */
-	g_debug ("schedule idle running %s", item->tid);
 	pk_transaction_set_state (item->transaction, PK_TRANSACTION_STATE_RUNNING);
 
 	/* add this idle, so that we don't have a deep out-of-order callchain */
@@ -499,7 +507,6 @@ pk_transaction_list_transaction_finished_cb (PkTransaction *transaction,
 					     PkTransactionList *tlist)
 {
 	gboolean ret;
-	guint timeout;
 	PkTransactionItem *item;
 	PkTransactionState state;
 	PkBackendJob *job;
@@ -527,7 +534,7 @@ pk_transaction_list_transaction_finished_cb (PkTransaction *transaction,
 
 		g_debug ("transaction finished and requires lock now, attempt %i", item->tries);
 
-		if (item->tries > 4) {
+		if (item->tries > PK_TRANSACTION_LIST_MAX_LOCK_RETRIES) {
 			/* fail the transaction */
 			job = pk_transaction_get_backend_job (item->transaction);
 
@@ -545,8 +552,6 @@ pk_transaction_list_transaction_finished_cb (PkTransaction *transaction,
 			g_source_remove (item->commit_id);
 			item->commit_id = 0;
 		}
-
-		g_debug ("transaction %s completed, marking finished", item->tid);
 		ret = pk_transaction_set_state (item->transaction, PK_TRANSACTION_STATE_FINISHED);
 		if (!ret) {
 			g_warning ("transaction could not be set finished!");
@@ -554,8 +559,7 @@ pk_transaction_list_transaction_finished_cb (PkTransaction *transaction,
 		}
 
 		/* give the client a few seconds to still query the runner */
-		timeout = pk_conf_get_int (tlist->priv->conf, "TransactionKeepFinishedTimeout");
-		item->remove_id = g_timeout_add_seconds (timeout, (GSourceFunc) pk_transaction_list_remove_item_cb, item);
+		item->remove_id = g_timeout_add_seconds (PK_TRANSACTION_KEEP_FINISHED_TIMOUT, (GSourceFunc) pk_transaction_list_remove_item_cb, item);
 		g_source_set_name_by_id (item->remove_id, "[PkTransactionList] remove");
 	}
 
@@ -567,7 +571,6 @@ pk_transaction_list_transaction_finished_cb (PkTransaction *transaction,
 	}
 
 	/* we have changed what is running */
-	g_debug ("emmitting ::changed");
 	g_signal_emit (tlist, signals [PK_TRANSACTION_LIST_CHANGED], 0);
 }
 
@@ -636,7 +639,7 @@ pk_transaction_list_create (PkTransactionList *tlist,
 	item = g_new0 (PkTransactionItem, 1);
 	item->list = g_object_ref (tlist);
 	item->tid = g_strdup (tid);
-	item->transaction = pk_transaction_new ();
+	item->transaction = pk_transaction_new (tlist->priv->introspection);
 	item->finished_id =
 		g_signal_connect_after (item->transaction, "finished",
 					G_CALLBACK (pk_transaction_list_transaction_finished_cb), tlist);
@@ -681,7 +684,6 @@ pk_transaction_list_create (PkTransactionList *tlist,
 
 	/* find out the number of transactions this uid already has in progress */
 	count = pk_transaction_list_get_number_transactions_for_uid (tlist, item->uid);
-	g_debug ("uid=%i, count=%i", item->uid, count);
 
 	/* would this take us over the maximum number of requests allowed */
 	max_count = pk_conf_get_int (tlist->priv->conf, "SimultaneousTransactionsForUid");
@@ -701,7 +703,7 @@ pk_transaction_list_create (PkTransactionList *tlist,
 	item->commit_id = g_timeout_add_seconds (timeout, (GSourceFunc) pk_transaction_list_no_commit_cb, item);
 	g_source_set_name_by_id (item->commit_id, "[PkTransactionList] commit");
 
-	g_debug ("adding transaction %p, item %p", item->transaction, item);
+	g_debug ("adding transaction %p", item->transaction);
 	g_ptr_array_add (tlist->priv->array, item);
 out:
 	return ret;
@@ -825,7 +827,7 @@ pk_transaction_list_commit (PkTransactionList *tlist, const gchar *tid)
 		return FALSE;
 	}
 
-	g_debug ("marking transaction %s as committed", item->tid);
+	/* marking transaction as committed */
 	ret = pk_transaction_set_state (item->transaction,
 					PK_TRANSACTION_STATE_COMMITTED);
 	if (!ret) {
@@ -844,23 +846,17 @@ pk_transaction_list_commit (PkTransactionList *tlist, const gchar *tid)
 	}
 
 	/* we will changed what is running */
-	g_debug ("emitting ::changed");
 	g_signal_emit (tlist, signals [PK_TRANSACTION_LIST_CHANGED], 0);
 
 	/* is one of the current running transactions background, and this new
 	 * transaction foreground? */
-	ret = pk_conf_get_bool (tlist->priv->conf,
-				"CancelBackgroundTransactions");
-	if (ret) {
-		if (!item->background && pk_transaction_list_get_background_running (tlist)) {
-			g_debug ("cancelling running background transactions and instead running %s",
-				item->tid);
-			pk_transaction_list_cancel_background (tlist);
-		}
+	if (!item->background && pk_transaction_list_get_background_running (tlist)) {
+		g_debug ("cancelling running background transactions and instead running %s",
+			item->tid);
+		pk_transaction_list_cancel_background (tlist);
 	}
 
 	/* do the transaction now, if possible */
-	g_debug ("running %s", item->tid);
 	if (pk_transaction_is_exclusive (item->transaction) == FALSE ||
 	    pk_transaction_list_get_exclusive_running (tlist) == 0)
 		pk_transaction_list_run_item (tlist, item);
@@ -1004,7 +1000,6 @@ pk_transaction_list_is_consistent (PkTransactionList *tlist)
 		goto out;
 
 	/* get state */
-	g_debug ("checking consistency as length %i", length);
 	for (i = 0; i < length; i++) {
 		item = (PkTransactionItem *) g_ptr_array_index (tlist->priv->array, i);
 		state = pk_transaction_get_state (item->transaction);
@@ -1021,31 +1016,35 @@ pk_transaction_list_is_consistent (PkTransactionList *tlist)
 			unknown_role++;
 	}
 
-	/* debug */
-	pk_transaction_list_print (tlist);
-
 	/* role not set */
-	if (unknown_role != 0)
+	if (unknown_role != 0) {
+		pk_transaction_list_print (tlist);
 		g_debug ("%i have an unknown role (CreateTransaction then nothing?)", unknown_role);
+	}
 
 	/* some are not committed */
-	if (no_commit != 0)
+	if (no_commit != 0) {
+		pk_transaction_list_print (tlist);
 		g_debug ("%i have not been committed and may be pending auth", no_commit);
+	}
 
 	/* more than one running */
 	if (running > 0) {
+		pk_transaction_list_print (tlist);
 		g_debug ("%i are running", running);
 	}
 
 	/* more than one exclusive transactions running? */
 	running_exclusive = pk_transaction_list_get_exclusive_running (tlist);
 	if (running_exclusive > 1) {
+		pk_transaction_list_print (tlist);
 		g_warning ("%i exclusive transactions running", running_exclusive);
 		ret = FALSE;
 	}
 
 	/* nothing running */
 	if (waiting == length) {
+		pk_transaction_list_print (tlist);
 		g_warning ("everything is waiting!");
 		ret = FALSE;
 	}
@@ -1063,7 +1062,7 @@ pk_transaction_list_wedge_check2 (PkTransactionList *tlist)
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_LIST (tlist), FALSE);
 
-	g_debug ("checking consistency a second time, as the first was not valid");
+	g_debug ("checking consistency a second time");
 	ret = pk_transaction_list_is_consistent (tlist);
 	if (ret) {
 		g_debug ("panic over");
@@ -1159,6 +1158,8 @@ pk_transaction_list_init (PkTransactionList *tlist)
 	tlist->priv = PK_TRANSACTION_LIST_GET_PRIVATE (tlist);
 	tlist->priv->conf = pk_conf_new ();
 	tlist->priv->array = g_ptr_array_new ();
+	tlist->priv->introspection = pk_load_introspection (PK_DBUS_INTERFACE_TRANSACTION ".xml",
+							    NULL);
 	tlist->priv->unwedge2_id = 0;
 	tlist->priv->unwedge1_id = g_timeout_add_seconds (PK_TRANSACTION_WEDGE_CHECK,
 							  (GSourceFunc) pk_transaction_list_wedge_check1, tlist);
@@ -1187,6 +1188,7 @@ pk_transaction_list_finalize (GObject *object)
 
 	g_ptr_array_foreach (tlist->priv->array, (GFunc) pk_transaction_list_item_free, NULL);
 	g_ptr_array_free (tlist->priv->array, TRUE);
+	g_dbus_node_info_unref (tlist->priv->introspection);
 	g_object_unref (tlist->priv->conf);
 	if (tlist->priv->plugins != NULL)
 		g_ptr_array_unref (tlist->priv->plugins);
